
package provide jtagRoutines 1.0
set DEBUG 1


###################################################################
# Append data to data string
###################################################################
proc append_data {dlength dstring d} {
  set pre ""
  if {$dstring == ""} {set dstring 0 }
  set app [expr 0x$dstring  | [expr $d << $dlength]]

  set dlength [expr $dlength +1]
  set tt  [expr $dlength /4]
  set rem [expr ([expr  $dlength % 4] == 0 )?0:1]
  set len [expr $tt + $rem]

  set app [format %x $app]
  set diff [expr $len - [string length $app]]
 
  for {set i [expr $diff]} {$i > 0} {set i [expr $i-1]} {
      set pre 0$pre
  }

  return $pre$app
}

###################################################################
# Right shift a data string by "shift" bits
###################################################################
proc rshift_data {dstring shift} {
  set pre ""
  set temp [expr 0x$dstring >> $shift] 
  set temp [format %x $temp]
  set bigshift [expr $shift / 4]
  set len [expr [string length $dstring] - $bigshift]

  set diff [expr $len - [string length $temp]]
  for {set i [expr $diff]} {$i > 0} {set i [expr $i-1]} {
      set pre 0$pre
  }  
  return $pre$temp 
}

###################################################################
# Send one bit out TMS and TDI
###################################################################
proc send_jtag_bit {tms tdi} {
  global DEBUG
  set send_value [expr 16 + 4*$tms + $tdi]
  lpt_wrdata $send_value
  lpt_wrdata [expr $send_value + 2]
  if {[expr [lpt_rdstat] & 16] != 0} {
    set tdo 1
  } else {
    set tdo 0
  }
  if {$DEBUG > 2} {puts "\t\tTMS = $tms TDI = $tdi TDO $tdo"}
  return $tdo
}
        
###################################################################
# Send a series of bits out TDI and TMS
###################################################################
proc send_jtag {length tms_string tdi_string} {
  global DEBUG
  if {$DEBUG > 1} {puts "\tsend_jtag $length $tms_string $tdi_string"}

  set tms_hex 0x$tms_string
  set tdi_hex 0x$tdi_string
  set return_value ""
  set slength 0
  set str_length [expr 4 * [string length $tms_string] ]
  for {set i [expr $str_length -1]} {$i >= $str_length-$length} {set i [expr $i -1]} {
    set mask_bit [expr 1 << $slength]
    set tms  [expr ( [expr $tms_hex & $mask_bit] == 0 )?0:1] 
    #set tms [get_bit $i $tms_string]
    set tdi  [expr ( [expr $tdi_hex & $mask_bit] == 0 )?0:1] 
    #set tdi [get_bit $i $tdi_string]
    set tdo [send_jtag_bit $tms $tdi]
    set return_value [append_data $slength $return_value $tdo]
    incr slength
  }
  if {$DEBUG > 1} {puts "\tsend_jtag returned $return_value"}
  return $return_value
}

###################################################################
# Set the instruction register 
###################################################################
proc sir {length data} {
  global DEBUG;
  if {$DEBUG > 0} {puts "Set instruction register $length $data"}
  send_jtag 4 "03" "00"
  set tms_string ""
  for {set i 0} {$i < [expr ceil($length/4.0)]} {incr i} {
     set tms_string "0$tms_string"
  }
  send_jtag [expr $length -1] $tms_string $data
  send_jtag 1 "1" [rshift_data $data [expr $length-1]]
  send_jtag 2 "01" "00"
}

###################################################################
# Set the data register
###################################################################
proc sdr {length data} {
  global DEBUG
  #move to shift ir
  if {$DEBUG > 0} {puts "Set data register $length $data"}
  send_jtag 7 "010" "000"

  #build tms string 
  set tms_string ""
  for {set i 0} {$i < [expr ceil($length/4.0)]} {incr i} {
     set tms_string "0$tms_string"
  }

  #send 1 bit (because we assume 1 pipeline delay)
  send_jtag 1 $tms_string $data
  set data [rshift_data $data 1]
  set tms_string [rshift_data $tms_string 1]

  #send other bits and capture data
  set return_value [send_jtag [expr $length-1] $tms_string $data]

  #shift out on last bit to finish collecting data
  set return_value [append_data [expr $length-1] $return_value [send_jtag 1 "1" "0"]]
  
  #return to idle
  send_jtag 3 "1" "0"
  return $return_value
}

###################################################################
# Send the instruction register 
###################################################################
proc get_device_id { } {
  sir 6 "09"
  set idcode [sdr 32 "000000000"]
  puts "IDCODE is $idcode"
}


###################################################################
# Enter user mode
###################################################################
proc enter_user_mode { } {
  sir 6 "02"
}

###################################################################
# Read a register
###################################################################
proc get_jtag {  } {
  set jtag_bytes "000"
  while {[expr "0x$jtag_bytes" & 2048] == 0} {
    set jtag_bytes [sdr 12 "000"]
  }
  set jtag_bytes [sdr 12 "100"]
  set jtag_bytes [string toupper [string range $jtag_bytes 1 2]]
  return $jtag_bytes
}

###################################################################
# Read a register
###################################################################
proc put_jtag { data } {
  for {set i 0} {$i < [string length $data]} {set i [expr $i+2]} {
    set string_piece [string range $data $i [expr $i+1]]
    sdr 12 "2$string_piece"
  }
}


###################################################################
# Send the instruction register 
###################################################################
proc reset_tap { } {
  send_jtag 8 "FF" "00"
}

##################################################################
# Send the instruction register 
###################################################################
proc idle_tap { } {
  send_jtag 1 "0" "0"
}

