
###################################################################
#  
###################################################################

lappend auto_path .

#load lpttcl

package require crc16 1.1.1
package require jtagRoutines 1.0

#Set to a 1 to talk to the PicoBlaze through JTAG
#Set to a 0 to talk to the PicoBlaze through RS232
set USE_JTAG          0

set flash_device "M25P16"
set flash_location "external"
set fpga   "3S700AN"
set prompt "%"
set MB_DEBUG 0
set FLASH_TYPE        0
set PAGE_SIZE         0
set SECTOR_SIZE       0
set BLOCK_SIZE        0
set PAGE_ADDER        0
set SECTOR_ADDER      0
set BLOCK_ADDER       0

switch $fpga {
  "3S50AN"   {set BITSTREAM_SIZE  437312}
  "3S200AN"  {set BITSTREAM_SIZE 1196128}
  "3S400AN"  {set BITSTREAM_SIZE 1886560}
  "3S700AN"  {set BITSTREAM_SIZE 2732640}
  "3S1400AN" {set BITSTREAM_SIZE 4755296}
}

###################################################################
# Get the next larget power of two
###################################################################
proc nextPow2 { x } {
   set value [expr int(pow(2,int(ceil(log($x)/log(2)))))]
   return $value
}


###################################################################
# Read a register in the port space
###################################################################
proc read_reg_rs232 {fh reg} {
   global MB_DEBUG
   puts -nonewline $fh [string toupper "R$reg"]
   if {$MB_DEBUG} {puts "Sent R$reg"}
   set value [gets $fh]
   if {$MB_DEBUG} {puts "Rcvd $value"}
   return $value
}


###################################################################
# Read a scratch pad value
###################################################################
proc read_scratch_rs232 {fh reg} {
   global MB_DEBUG
   puts -nonewline $fh [string toupper "S$reg"]
   if {$MB_DEBUG} {puts "Sent S$reg"}
   set value [gets $fh]
   if {$MB_DEBUG} {puts "Rcvd $value"}
   return $value
}

###################################################################
# Write a register in the port space
###################################################################
proc write_reg_rs232 {fh reg data} {
   global MB_DEBUG
   puts -nonewline $fh [string toupper "W$reg$data"]
   if {$MB_DEBUG} {puts "Sent W$reg$data"}
   return
}

###################################################################
# Burst a register in the port space
###################################################################
proc burst_reg_rs232 {fh reg data} {
   global MB_DEBUG
   set size [format %04x [expr ([string length $data] / 2) % 65536]]
   puts -nonewline $fh [string toupper "B$reg$size$data"]
   if {$MB_DEBUG} {puts "Sent B$reg$size$data"}
   return
}

###################################################################
# Have the PicoBlaze calculate crc for a bitstream
###################################################################
proc calc_crc_rs232 {fh bsNum} {
   global MB_DEBUG
   set bsNum [format %02x $bsNum]
   puts -nonewline $fh [string toupper "C$bsNum"]
   if {$MB_DEBUG} {puts "Sent C$bsNum"}
   set value [gets $fh]
   if {$MB_DEBUG} {puts "Rcvd $value"}
   return $value
}

###################################################################
# Read a register in the port space
###################################################################
proc read_reg_jtag {reg} {
   global MB_DEBUG
   put_jtag  [string toupper "52$reg"]
   if {$MB_DEBUG} {puts "Sent R$reg"}
   set value [get_jtag]
   if {$MB_DEBUG} {puts "Rcvd $value"}
   return $value
}


###################################################################
# Read a scratch pad value
###################################################################
proc read_scratch_jtag {reg} {
   global MB_DEBUG
   put_jtag [string toupper "53$reg"]
   if {$MB_DEBUG} {puts "Sent S$reg"}
   set value [get_jtag]
   if {$MB_DEBUG} {puts "Rcvd $value"}
   return $value
}

###################################################################
# Write a register in the port space
###################################################################
proc write_reg_jtag {reg data} {
   global MB_DEBUG
   put_jtag [string toupper "57$reg$data"]
   if {$MB_DEBUG} {puts "Sent W$reg$data"}
   return
}

###################################################################
# Burst a register in the port space
###################################################################
proc burst_reg_jtag {reg data} {
   global MB_DEBUG
   set size [format %04x [expr ([string length $data] / 2) % 65536]]
   put_jtag [string toupper "42$reg$size$data"]
   if {$MB_DEBUG} {puts "Sent B$reg$size$data"}
   return
}

###################################################################
# Have the PicoBlaze calculate crc for a bitstream
###################################################################
proc calc_crc_jtag {bsNum} {
   global MB_DEBUG
   set bsNum [format %02x $bsNum]
   put_jtag [string toupper "43$bsNum"]
   if {$MB_DEBUG} {puts "Sent C$bsNum"}
   set value "[get_jtag][get_jtag]"
   if {$MB_DEBUG} {puts "Rcvd $value"}
   return $value
}

###################################################################
# Read a register in the port space
###################################################################
proc read_reg {fh reg} {
   global USE_JTAG
   if {$USE_JTAG == 1} {
     return [read_reg_jtag $reg]
   } else {
     return [read_reg_rs232 $fh $reg]
   }
}


###################################################################
# Read a scratch pad value
###################################################################
proc read_scratch {fh reg} {
   global USE_JTAG
   if {$USE_JTAG == 1} {
     return [read_scratch_jtag $reg]
   } else {
     return [read_scratch_rs232 $fh $reg]
   }
}

###################################################################
# Write a register in the port space
###################################################################
proc write_reg {fh reg data} {
   global USE_JTAG
   if {$USE_JTAG == 1} {
     return [write_reg_jtag $reg $data]
   } else {
     return [write_reg_rs232 $fh $reg $data]
   }
}

###################################################################
# Burst a register in the port space
###################################################################
proc burst_reg {fh reg data} {
   global USE_JTAG
   if {$USE_JTAG == 1} {
     return [burst_reg_jtag $reg $data]
   } else {
     return [burst_reg_rs232 $fh $reg $data]
   }
}

###################################################################
# Have the PicoBlaze calculate crc for a bitstream
###################################################################
proc calc_crc {fh bsNum} {
   global USE_JTAG
   if {$USE_JTAG == 1} {
     return [calc_crc_jtag $bsNum]
   } else {
     return [calc_crc_rs232 $fh $bsNum]
   }
}

###################################################################
# Investigate which flash devices is being used
###################################################################
proc discover_flash_device { fh } {
  global flash_device
  global flash_location
  global FLASH_TYPE
  global PAGE_SIZE
  global SECTOR_SIZE
  global BLOCK_SIZE
  global PAGE_ADDER
  global SECTOR_ADDER
  global BLOCK_ADDER

  write_reg $fh "30" "00"
  write_reg $fh "31" "9F"
  write_reg $fh "31" "00"
  set mfg_id [read_reg  $fh "31"]
  if {$mfg_id == "20"} {
    set flash_device "M25P16"
    set flash_location "external" 
  } else {
    write_reg $fh "31" "00"
    set size [read_reg $fh "31"]
    switch $size {
      "22" { set flash_device "AT45DB011D" }
      "24" { set flash_device "AT45DB041D" }
      "25" { set flash_device "AT45DB081D" }
      "26" { set flash_device "AT45DB161D" }
    }
    set fw_cfg_reg "50"
    if {[expr "0x[read_reg $fh $fw_cfg_reg]" & 4]} {
      set flash_location "internal"
    } else {
      set flash_location "external"
    }
  }
  write_reg $fh "30" "FF"

  switch $flash_device {
    "M25P16" {
       set FLASH_TYPE        0
       set PAGE_SIZE         256
       set SECTOR_SIZE       65536 }
  
    "AT45DB011D" {
       set FLASH_TYPE        1
       set PAGE_SIZE         264
       set SECTOR_SIZE       33792 }
       
    "AT45DB041D" {
       set FLASH_TYPE        1
       set PAGE_SIZE         264
       set SECTOR_SIZE       67584 }
  
    "AT45DB081D" {
       set FLASH_TYPE        1
       set PAGE_SIZE         264
       set SECTOR_SIZE       67584 }
  
    "AT45DB161D" {
       set FLASH_TYPE        1
       set PAGE_SIZE         528
       set SECTOR_SIZE       135168 }
  }

  set BLOCK_SIZE [expr 8*$PAGE_SIZE]
  set PAGE_ADDER   [nextPow2 $PAGE_SIZE]
  set SECTOR_ADDER [nextPow2 $SECTOR_SIZE]
  set BLOCK_ADDER  [nextPow2 $BLOCK_SIZE]

}

###################################################################
# Read the fpga version from the registers
###################################################################
proc read_fpga_version {fh} {
  set rev_lsb [read_reg $fh "00"]
  set rev_msb [read_reg $fh "01"]
  return "$rev_msb$rev_lsb"
}

###################################################################
# Read the prompt from the scratchpad
###################################################################
proc read_prompt {fh} {
  set value [read_scratch $fh "38"]
  return $value
}

###################################################################
# Read the SPI manufacturer id from the scratchpad
###################################################################
proc read_mfg_id {fh} {
  set value [read_scratch $fh "03"]
  return $value
}


###################################################################
# Read the number of bs ffrom the scratch pad
###################################################################
proc read_num_bs {fh} {
  set value [read_reg $fh "70"]
  return $value
}

###################################################################
# Read the bitstream table to get field $field for bitstream $bs
###################################################################
proc read_bs_table {fh bs field} {
  # read the valid word
  set scratch_loc [expr 8*$bs + 2*$field]
  set value_h [read_scratch $fh [format %02x $scratch_loc]]
  set scratch_loc [expr 8*$bs + 2*$field +1]
  set value_l [read_scratch $fh [format %02x $scratch_loc]]
  set value "$value_h$value_l"
  return $value
}

###################################################################
# Read the most recent history byte
###################################################################
proc read_history {fh} {
  set value [read_scratch $fh "36"]
  return $value
}

###################################################################
# Read the sorted revision order 
###################################################################
proc read_order {fh} {
  set first [read_scratch $fh "30"]
  set second [read_scratch $fh "31"]
  set third [read_scratch $fh "32"]
  return "$first $second $third"
}

###################################################################
# Read the boot decision
###################################################################
proc read_boot_decision {fh} {
  set value [read_scratch $fh "34"]
}

###################################################################
# Enable spi writes
###################################################################
proc enable_spi_writes {fh} {
  write_reg $fh "30" "00"
  write_reg $fh "31" "06"
  write_reg $fh "30" "FF"
}

###################################################################
# Disable spi writes
###################################################################
proc disable_spi_writes {fh} {
  write_reg $fh "30" "00"
  write_reg $fh "31" "04"
  write_reg $fh "30" "FF"
}

###################################################################
# Wait for spi action to complete
###################################################################
proc wait_for_wip_done {fh} {
  write_reg $fh "30" "00"
  write_reg $fh "31" "05"
  set wip 1
  set spi_dreg "31"
  while {$wip == 1} {
    write_reg $fh $spi_dreg "00"
    set wip [expr "0x[read_reg $fh $spi_dreg]" & 1]
    flush stdout
  }
  write_reg $fh "30" "FF"
}

###################################################################
# Wait for spi action to complete
###################################################################
proc wait_for_busy_done {fh} {
  write_reg $fh "30" "00"
  write_reg $fh "31" "D7"
  set ready 0
  set spi_dreg "31"
  while {$ready == 0} {
    write_reg $fh $spi_dreg "00"
    set ready [expr [format %d "0x[read_reg $fh $spi_dreg]"] & 128]
    flush stdout
  }
  write_reg $fh "30" "FF"
}

###################################################################
# Reset the PicoBlaze
###################################################################
proc reset_picoblaze {fh} {
  write_reg $fh "FF" "FF"
}

###################################################################
# Corrupt a spi byte
###################################################################
proc corrupt_spi_byte {fh} {
  puts -nonewline "Enter the address of the byte (hex) : "
  flush stdout
  gets stdin inputString
  set inputString [string toupper $inputString]
  set addr_hi  [string range $inputString 0 1]
  set addr_mid [string range $inputString 2 3]
  set addr_lo  [string range $inputString 4 5]
  puts -nonewline "Enter the value for the write (hex) : "
  flush stdout
  gets stdin inputString 
  set data [string range $inputString 0 1]
  enable_spi_writes $fh
  write_reg $fh "30" "00"
  write_reg $fh "31" "02"
  write_reg $fh "31" $addr_hi
  write_reg $fh "31" $addr_mid
  write_reg $fh "31" $addr_lo
  write_reg $fh "31" $data
  write_reg $fh "30" "FF"
}

###################################################################
# Boot from a bitstream image at a given address
###################################################################
proc boot_from_bs {fh} {
  puts -nonewline "Enter the address of the bitstream (hex) : "
  flush stdout
  gets stdin inputString
  set inputString [string toupper $inputString]
  set addr_hh  "00"
  set addr_hi  [string range $inputString 0 1]
  set addr_mid [string range $inputString 2 3]
  set addr_lo  [string range $inputString 4 5]
  write_reg $fh "40" $addr_lo
  write_reg $fh "41" $addr_mid
  write_reg $fh "42" $addr_hi
  write_reg $fh "43" $addr_hh
  write_reg $fh "47" $addr_hh
}
###################################################################
# Dump a spi page
###################################################################
proc dump_spi_page {fh} {
  global PAGE_SIZE
  puts -nonewline "Enter the address of the page (hex) : "
  flush stdout
  gets stdin inputString
  set inputString [string toupper $inputString]
  set addr_hi  [string range $inputString 0 1]
  set addr_mid [string range $inputString 2 3]
  set addr_lo  [string range $inputString 4 5]
  write_reg $fh "30" "00"
  write_reg $fh "31" "03"
  write_reg $fh "31" $addr_hi
  write_reg $fh "31" $addr_mid
  write_reg $fh "31" $addr_lo
  for {set row 0} {$row < [expr $PAGE_SIZE / 16.0]} {incr row} {
    #puts -nonewline "[expr int (16.0 * $row)] " 
    for {set col 0} {$col < 16} {incr col} {
      write_reg $fh "31" "00"
      puts -nonewline [read_reg $fh "31"]
      puts -nonewline " "
    }
    puts ""
  }
  write_reg $fh "30" "FF"
}

###################################################################
# Erase a spi page
###################################################################
proc erase_spi_page_atmel {fh address} {
  set addr_hex [format %06x $address]    
  set addr_hi  [string range $addr_hex 0 1]
  set addr_mid [string range $addr_hex 2 3]
  set addr_lo  [string range $addr_hex 4 5]
  write_reg $fh "30" "00"
  write_reg $fh "31" "81"
  write_reg $fh "31" $addr_hi
  write_reg $fh "31" $addr_mid
  write_reg $fh "31" $addr_lo
  write_reg $fh "30" "FF"
  wait_for_busy_done $fh
}

###################################################################
# Erase a block
###################################################################
proc erase_spi_block_atmel {fh address} {
  set addr_hex [format %06x $address]    
  set addr_hi  [string range $addr_hex 0 1]
  set addr_mid [string range $addr_hex 2 3]
  set addr_lo  [string range $addr_hex 4 5]
  write_reg $fh "30" "00"
  write_reg $fh "31" "50"
  write_reg $fh "31" $addr_hi
  write_reg $fh "31" $addr_mid
  write_reg $fh "31" $addr_lo
  write_reg $fh "30" "FF"
  wait_for_busy_done $fh
}

###################################################################
# Erase a spi sector
###################################################################
proc erase_spi_sector_atmel {fh address} {
  set addr_hex [format %06x $address]    
  set addr_hi  [string range $addr_hex 0 1]
  set addr_mid [string range $addr_hex 2 3]
  set addr_lo  [string range $addr_hex 4 5]
  write_reg $fh "30" "00"
  write_reg $fh "31" "7C"
  write_reg $fh "31" $addr_hi
  write_reg $fh "31" $addr_mid
  write_reg $fh "31" $addr_lo
  write_reg $fh "30" "FF"
  wait_for_busy_done $fh
}

##################################################################
# Erase a spi sector
###################################################################
proc erase_spi_sector_st {fh address} {
  set addr_hex [format %06x $address]    
  set addr_hi  [string range $addr_hex 0 1]
  set addr_mid [string range $addr_hex 2 3]
  set addr_lo  [string range $addr_hex 4 5]
  enable_spi_writes $fh
  write_reg $fh "30" "00"
  write_reg $fh "31" "D8"
  write_reg $fh "31" $addr_hi
  write_reg $fh "31" $addr_mid
  write_reg $fh "31" $addr_lo
  write_reg $fh "30" "FF"
  wait_for_wip_done $fh
}

###################################################################
# Erase a history 
###################################################################
proc erase_history {fh} {
  global FLASH_TYPE
  set history_addr "0x[read_reg $fh 76][read_reg $fh 77]00"
  if {$FLASH_TYPE == 1} {
    erase_spi_page_atmel $fh $history_addr
  } else {
    erase_spi_sector_st $fh $history_addr      
  }
}

###################################################################
# Erase a bit stream for atmel devices
###################################################################
proc erase_bs_atmel {fh bsNum} {
  global BITSTREAM_SIZE
  global PAGE_SIZE
  global SECTOR_SIZE
  global BLOCK_SIZE
  global PAGE_ADDER
  global SECTOR_ADDER
  global BLOCK_ADDER

  if {$bsNum > 0} {
    set bs_addr_reg "7$bsNum"
    set addr "0x[read_reg $fh $bs_addr_reg]0000"
  } else {
    set addr "000000"
  }
  set bytes_left [expr int($BITSTREAM_SIZE/8)]
  while {$bytes_left >= $SECTOR_SIZE} {
    erase_spi_sector_atmel $fh $addr
    set addr [expr $addr + $SECTOR_ADDER]
    set bytes_left [expr $bytes_left - $SECTOR_SIZE]
  }
  while {$bytes_left >= $BLOCK_SIZE} {
    erase_spi_block_atmel $fh $addr
    set addr [expr $addr + $BLOCK_ADDER]
    set bytes_left [expr $bytes_left - $BLOCK_SIZE]
  }
  while {$bytes_left >= $PAGE_SIZE} {
    erase_spi_page_atmel $fh $addr
    set addr [expr $addr + $PAGE_ADDER]
    set bytes_left [expr $bytes_left - $PAGE_SIZE]
  }
}

###################################################################
# Erase a bit stream st devices
###################################################################
proc erase_bs_st {fh bsNum} {
  global BITSTREAM_SIZE
  global SECTOR_SIZE
  if {$bsNum > 0} {
    set bs_addr_reg "7$bsNum"
    set start_addr "0x[read_reg $fh $bs_addr_reg]0000"
  } else {
    set start_addr "0x000000"
  }
  set last_addr [format %d [expr $start_addr + $BITSTREAM_SIZE/8 - 1]]
  if {$bsNum > 0} {
    set last_addr [expr $last_addr +16]
  }
  for {set addr $start_addr} {$addr < $last_addr} {set addr [expr $addr + $SECTOR_SIZE]} {
    erase_spi_sector_st $fh $addr
  }
}

###################################################################
# Erase a bit stream st devices
###################################################################
proc erase_bs {fh bsNum} {
  global FLASH_TYPE
  if {$FLASH_TYPE == 1} {
    erase_bs_atmel $fh $bsNum
  } else {
    erase_bs_st $fh $bsNum
  }
}

###################################################################
# Erase a bit stream
###################################################################
proc erase_bit_stream {fh} {
  puts -nonewline "Enter the bitstream number : "
  flush stdout
  gets stdin bsNum
  erase_bs $fh $bsNum
}

###################################################################
# Page to buffer for atmel devices
###################################################################
proc page_to_buffer_atmel {fh addr} {
  set addr_hex [format %06x $addr]    
  set addr_hi  [string range $addr_hex 0 1]
  set addr_mid [string range $addr_hex 2 3]
  set addr_lo  [string range $addr_hex 4 5]
  write_reg $fh "30" "00"
  write_reg $fh "31" "53"
  write_reg $fh "31" $addr_hi
  write_reg $fh "31" $addr_mid
  write_reg $fh "31" $addr_lo
  write_reg $fh "30" "FF"
  wait_for_busy_done $fh
}

###################################################################
# Write buffer for atmel devices
###################################################################
proc write_buffer_atmel {fh addr bytes} {
  global PAGE_SIZE
  global PAGE_ADDER
  set addr_hex [format %06x [expr $addr % $PAGE_ADDER]]
  set addr_hi  [string range $addr_hex 0 1]
  set addr_mid [string range $addr_hex 2 3]
  set addr_lo  [string range $addr_hex 4 5]
  set lgth [string length $bytes]
  write_reg $fh "30" "00"
  write_reg $fh "31" "84"
  write_reg $fh "31" $addr_hi
  write_reg $fh "31" $addr_mid
  write_reg $fh "31" $addr_lo
  burst_reg $fh "31" $bytes
  write_reg $fh "30" "FF"
}

###################################################################
# Buffer to page program for atmel devices
###################################################################
proc buffer_to_page_atmel {fh addr} {
  set addr_hex [format %06x $addr]    
  set addr_hi  [string range $addr_hex 0 1]
  set addr_mid [string range $addr_hex 2 3]
  set addr_lo  [string range $addr_hex 4 5]
  write_reg $fh "30" "00"
  write_reg $fh "31" "88"
  write_reg $fh "31" $addr_hi
  write_reg $fh "31" $addr_mid
  write_reg $fh "31" $addr_lo
  write_reg $fh "30" "FF"
  wait_for_busy_done $fh
}

###################################################################
# Write a page for atmel devices
###################################################################
proc write_page_atmel {fh addr bytes} {
  page_to_buffer_atmel $fh $addr
  write_buffer_atmel $fh $addr $bytes
  buffer_to_page_atmel $fh $addr
}

###################################################################
# Write a page for st devices
###################################################################
proc write_page_st {fh addr bytes} {
  set addr_hex [format %06x $addr]    
  set addr_hi  [string range $addr_hex 0 1]
  set addr_mid [string range $addr_hex 2 3]
  set addr_lo  [string range $addr_hex 4 5]
  enable_spi_writes $fh
  write_reg $fh "30" "00"
  write_reg $fh "31" "02"
  write_reg $fh "31" $addr_hi
  write_reg $fh "31" $addr_mid
  write_reg $fh "31" $addr_lo
  burst_reg $fh "31" $bytes
  write_reg $fh "30" "FF"
  wait_for_wip_done $fh
}

###################################################################
# Write a page
###################################################################
proc write_page {fh addr bytes} {
  global FLASH_TYPE
  if {$FLASH_TYPE == 1} {
    write_page_atmel $fh $addr $bytes
  } else {
    write_page_st $fh $addr $bytes
  }
}

###################################################################
# Generate a crc16 
###################################################################
proc crc16_calc { startValue bytes} {
  set crc16_value $startValue
  set nextByteBin [binary format H* $bytes]
  set crc16_value [crc::crc-ccitt -seed $crc16_value "$nextByteBin"]
  return $crc16_value
}

###################################################################
# Calculate a crc for a given bitstream
###################################################################
proc get_calc_crc {fh} {
  puts ""
  puts -nonewline "Enter the bitstream number : "
  flush stdout
  gets stdin bsNum
  set result [calc_crc $fh $bsNum]
  puts "CRC result is 0x$result"
}
  
###################################################################
# Write a bit stream
###################################################################
proc write_bit_stream {fh} {
  global PAGE_SIZE
  global PAGE_ADDER
  puts -nonewline "Enter the bitstream number : "
  flush stdout
  gets stdin bsNum
  puts -nonewline "Enter the hex file name : "
  flush stdout
  gets stdin hexfileName
  set hexfh [open $hexfileName RDONLY]
  if {$bsNum > 0} {
    puts -nonewline "Enter the revision number (eg. 0101) : "
    flush stdout
    gets stdin revision
  }
  puts "\nErasing the bitstream location in flash ..."
  erase_bs $fh $bsNum
  if {$bsNum > 0} {
    set bs_addr_reg "7$bsNum"
    set base_addr "0x[read_reg $fh $bs_addr_reg]0000"
  } else {
    set base_addr "000000"
  }
  set numBytes [expr ([file size $hexfileName] /2) -1]
  set numBytes_hex [format %06x $numBytes]
  set numPages [expr int($numBytes/$PAGE_SIZE) +1]
  set numkB [expr $numBytes / 1024]
  puts "\nNeed to write $numkB kB"
  set kBSent 0
  puts ""
  set check_value 0xFFFF
  if {$bsNum > 0} {
    set addr [expr $base_addr + 16]
    set myBytes [string trim [read $hexfh [expr 2*($PAGE_SIZE-16)]]]
    write_page $fh $addr $myBytes
    set addr [expr $addr -16 + $PAGE_ADDER]
  } else {
    set addr $base_addr	   
    set myBytes [string trim [read $hexfh [expr 2*$PAGE_SIZE]]]
    write_page $fh $addr $myBytes
    set addr [expr $addr + $PAGE_ADDER]
  }
  set check_value [crc16_calc $check_value $myBytes]
  for {set page 1} {$page < $numPages} {incr page} {
    set myBytes [string trim [read $hexfh [expr 2*$PAGE_SIZE]]]
    set numBytes [expr [string length $myBytes] / 2]
    if {$numBytes > 0} {
      write_page $fh $addr $myBytes
      if {$bsNum > 0} {
        set check_value [crc16_calc $check_value $myBytes]
      }
      set addr [expr $addr + $PAGE_ADDER]
      set old_kBSent $kBSent
      set kBSent [expr int ($page*$PAGE_SIZE / 1024)]
      if {$kBSent > $old_kBSent} {
        puts -nonewline "\b\b\b\b\b\b\b\b\b\b\b\bSent $kBSent kB"
      }
    }
  }
  if {$bsNum > 0} {
    set check_value [string toupper [format %04x $check_value]]
    write_page $fh [expr $base_addr +2] "$revision$check_value$numBytes_hex"
    write_page $fh $base_addr "00FF"
  }
  puts ""
  close $hexfh
}

###################################################################
# Prototyping routine
###################################################################
proc proto_routine {fh} {
   write_reg "30" "00"
   write_reg "31" "53"
   write_reg "31" "3F"
   write_reg "31" "FC"
   write_reg "31" "00"
   write_reg "30" "FF"

   write_reg "30" "00"
   write_reg "31" "84"
   write_reg "31" "00"
   write_reg "31" "00"
   write_reg "31" "02"
   write_reg "31" "02"
   write_reg "30" "FF"
}

###################################################################
###################################################################
##
##
##  PROGRAM MAINLINE
##
##
###################################################################
###################################################################

if {$USE_JTAG == 0} {
  #Setup the serial port
  set fh [open COM1: RDWR]
  fconfigure $fh -blocking 1 -mode 115200,n,8,1 -translation binary -buffering none
} else {
  set fh 0
  reset_tap
  idle_tap
  get_device_id
  reset_tap
  idle_tap
  enter_user_mode
}

#Look for a prompt from the fpga
puts "\nTrying to connect to device ... \n";

set outer_loop_done 0

while {$outer_loop_done == 0} {

  set prompt [read_prompt $fh]
  if {[string match $prompt "25"] == 1} {
    set prompt "%"
  } 
  if {[string match $prompt "3E"] == 1} {
    set prompt ">"
  }

  if {[string match "%" $prompt] == 1} {
    puts -nonewline "Connected to bootstrap ";
  }
  if {[string match ">" $prompt] == 1} {
    puts -nonewline "Connected to application ";
  }
  
  puts "(revision 0x[read_fpga_version $fh])"

  discover_flash_device $fh
  set spi_mfg_id [read_mfg_id $fh]
  set spi_num_bs [format %d "0x[read_num_bs $fh]"]

  puts ""
  puts "Found $flash_location $flash_device"
  puts ""
  puts ""
  puts " PicoBlaze scratchpad information"
  puts " -------------------------------"
  puts " The SPI manufacturer id is 0x$spi_mfg_id"
  puts " There are $spi_num_bs possible application bitstreams"
  puts ""
  
  #Get the bs table
  for {set i 1} {$i <= $spi_num_bs} {incr i} {
    set bs_info($i.valid)    [read_bs_table $fh $i 0]
    set bs_info($i.revision) [read_bs_table $fh $i 1]
    set bs_info($i.check)    [read_bs_table $fh $i 2]
  }

  #Get the bs table
  for {set i [expr $spi_num_bs+1]} {$i < 4} {incr i} {
    set bs_info($i.valid)    "FFFF"
    set bs_info($i.revision) "FFFF"
    set bs_info($i.check)    "FFFF"
  }
  
  #Get the history value
  set hist_value [read_history $fh]
  if {[string match "FF" $hist_value] == 1} {
    set history "(none)"
  } else {
    set bs [string range $hist_value 0 0]
    set attempt_val [string range $hist_value 1 1]
    if {[string match "0" $attempt_val] == 1} {
      set history "bitstream $bs was successful"
    } else {
      if {[string match "E" $attempt_val] == 1} {set attempts 1}
      if {[string match "C" $attempt_val] == 1} {set attempts 2}
      if {[string match "8" $attempt_val] == 1} {set attempts 3}
      set history "bitstream $bs attempted $attempts time(s)"
    }
  }
  
  #Get the revision order and boot decision
  set revision_order [read_order $fh]
  set boot_decision [string range [read_boot_decision $fh] 1 1]
  
  #Print the table
  puts " Bitstream     1     2    3"
  puts " ============================"
  puts " Valid       $bs_info(1.valid)  $bs_info(2.valid) $bs_info(3.valid)"
  puts " Revision    $bs_info(1.revision)  $bs_info(2.revision) $bs_info(3.revision)"
  puts " CheckValue  $bs_info(1.check)  $bs_info(2.check) $bs_info(3.check)"
  puts ""
  puts "Boot history: $history"
  puts "Revision order: $revision_order"
  puts "Boot decision: $boot_decision"
  
  set inner_loop_done 0
  while {$inner_loop_done == 0} {
    puts ""
    puts "Menu"
    puts "(W)rite a bit file"
    puts "(C)alc crc for a bitstream"
    puts "(E)rase a bit file"
    puts "Erase (H)istory"
    puts "(O)verwrite a spi byte"
    puts "(D)ump a spi page"
    puts "(B)oot from a bitstream"
    puts "(R)eset PicoBlaze"
    puts "(Q)uit"
    puts ""
    puts -nonewline "$prompt "
    flush stdout
    
    #Get the menu entry
    gets stdin inputString
    set inputString [string range $inputString 0 0]
   
    if {[string match [string toupper $inputString] "W"] == 1} {
      write_bit_stream $fh
    }
    if {[string match [string toupper $inputString] "C"] == 1} {
      get_calc_crc $fh
    }
    if {[string match [string toupper $inputString] "E"] == 1} {
      erase_bit_stream $fh
    }
    if {[string match [string toupper $inputString] "H"] == 1} {
      erase_history $fh
    }
    if {[string match [string toupper $inputString] "O"] == 1} {
      corrupt_spi_byte $fh
    }
    if {[string match [string toupper $inputString] "D"] == 1} {
      dump_spi_page $fh
    }
    if {[string match [string toupper $inputString] "P"] == 1} {
      proto_routine $fh
    }
    if {[string match [string toupper $inputString] "B"] == 1} {
      boot_from_bs $fh
      set outer_loop_done 1
      set inner_loop_done 1
    }
    if {[string match [string toupper $inputString] "R"] == 1} {
      reset_picoblaze $fh
      after 1000
      puts ""
      set inner_loop_done 1
    }
    if {[string match [string toupper $inputString] "T"] == 1} {
      if {$MB_DEBUG == 1} {
	set MB_DEBUG 0
      } else {
	set MB_DEBUG 1
      }
    }
    if {[string match [string toupper $inputString] "Q"] == 1} {
      set outer_loop_done 1
      set inner_loop_done 1
    }
  }
}

if {$USE_JTAG == 0} {
  close $fh
}

