//////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2006 Xilinx, Inc.
// This design is confidential and proprietary of Xilinx, All Rights Reserved.
//////////////////////////////////////////////////////////////////////////////
//   ____  ____
//  /   /\/   /
// /___/  \  /   Vendor:        Xilinx
// \   \   \/    Version:       1.0.0
//  \   \        Filename:      application2.v
//  /   /        Date Created:  October 20, 2006
// /___/   /\    Last Modified: October 08, 2007
// \   \  /  \
//  \___\/\___\
//
// Devices:   Spartan-3 Generation FPGA
// Purpose:   Failsafe multiboot  
// Contact:   jimwe@xilinx.com
// Reference: None
//
// Revision History:
//   Rev 1.0.0 - (jimwe) First created October 20, 2006.
//                       Reuse of crabill's Spartan3A boarddemo
//
//   Rev 1.1.0 - (jimwe) Added support for Spartan 3AN
//
//////////////////////////////////////////////////////////////////////////////
//
// LIMITED WARRANTY AND DISCLAIMER. These designs are provided to you "as is".
// Xilinx and its licensors make and you receive no warranties or conditions,
// express, implied, statutory or otherwise, and Xilinx specifically disclaims
// any implied warranties of merchantability, non-infringement, or fitness for
// a particular purpose. Xilinx does not warrant that the functions contained
// in these designs will meet your requirements, or that the operation of
// these designs will be uninterrupted or error free, or that defects in the
// designs will be corrected. Furthermore, Xilinx does not warrant or make any
// representations regarding use or the results of the use of the designs in
// terms of correctness, accuracy, reliability, or otherwise.
//
// LIMITATION OF LIABILITY. In no event will Xilinx or its licensors be liable
// for any loss of data, lost profits, cost or procurement of substitute goods
// or services, or for any special, incidental, consequential, or indirect
// damages arising from the use or operation of the designs or accompanying
// documentation, however caused and on any theory of liability. This
// limitation will apply even if Xilinx has been advised of the possibility
// of such damage. This limitation shall apply not-withstanding the failure
// of the essential purpose of any limited remedies herein.
//
//////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2006 Xilinx, Inc.
// This design is confidential and proprietary of Xilinx, All Rights Reserved.
//////////////////////////////////////////////////////////////////////////////

`timescale 1 ns / 1 ps

module application
  (
  input  wire        CLK_50MHZ,
  input  wire        RESET,
  input  wire        PAUSE,

  input  wire        RS232_DCE_RXD,
  output wire        RS232_DCE_TXD,

  input  wire        SPI_MISO,
  output wire        SPI_MOSI,
  output wire        SPI_SCK,
  output wire        SPI_SS_B,
  output wire        SPI_WP,

  output reg [7:0]   LD
  );

  // parameter SPI_DEVICE       = "M25P16";
  parameter SPI_DEVICE       = "AT45DB161D";
  parameter USE_REG          = "TRUE";
  parameter USE_JTAG         = "TRUE";
  parameter USE_RS232        = "TRUE";
  parameter CHECK_CRC        = "TRUE";
  parameter BOOTSTRAP_FW     = "FALSE";

  //******************************************************************//
  // Create global clock and asynchronous system reset.               //
  //******************************************************************//

  wire          clk_25mhz;
  wire          clknub;
  wire          locked;
  wire          reset_i;
  wire          debug_reset;
  
  DCM_SP DCM_SP_INST(
    .CLKIN(CLK_50MHZ),
    .CLKFB(clk_25mhz),
    .RST(1'b0),
    .PSEN(1'b0),
    .PSINCDEC(1'b0),
    .PSCLK(1'b0),
    .DSSEN(1'b0),
    .CLK0(clknub),
    .CLK90(),
    .CLK180(),
    .CLK270(),
    .CLKDV(),
    .CLK2X(),
    .CLK2X180(),
    .CLKFX(),
    .CLKFX180(),
    .STATUS(),
    .LOCKED(locked),
    .PSDONE()
  );
  defparam DCM_SP_INST.CLKIN_DIVIDE_BY_2 = "TRUE";
  defparam DCM_SP_INST.CLKIN_PERIOD = 20.000;

  BUFG BG_0 (.I(clknub), .O(clk_25mhz));

  synchro #(.INITIALIZE("LOGIC1"))
  synchro_reset (.async(!locked || debug_reset || RESET),
                 .sync(reset_i),
                 .clk(clk_25mhz));

  //******************************************************************//
  // Instantiate application code (racing LEDs)
  //******************************************************************//

  reg [20:0] clk_divide;
  reg        clk_enable;

  always @(posedge clk_25mhz)
  begin
    if (reset_i)
      begin
        clk_enable <= 0;
        clk_divide <= 21'h0;
      end
    else 
      begin
        clk_enable <= &clk_divide;
        clk_divide <= clk_divide +1;
      end
  end

  always @(posedge clk_25mhz)
  begin
    if (reset_i)
      LD <= 8'h01;
    else if (clk_enable)
      LD <= {LD[0],LD[7:1]};
  end

  //******************************************************************//
  // Instantiate picoController
  //******************************************************************//

  wire          spi_miso_i;
  wire          spi_mosi_i;
  wire          spi_sck_i;
  wire          spi_ss_b_i;

  picoController #( .SPI_DEVICE(SPI_DEVICE),
                    .USE_REG(USE_REG),
                    .USE_JTAG(USE_JTAG),
                    .USE_RS232(USE_RS232),
	   	              .CHECK_CRC(CHECK_CRC),
                    .BOOTSTRAP_FW(BOOTSTRAP_FW) )
  picoController_inst (
    .CLK(clk_25mhz),
    .RESET(reset_i),
    .PAUSE(PAUSE),
    .FPGA_REVISION(16'h0201),
    .RS232_DCE_RXD(RS232_DCE_RXD),
    .RS232_DCE_TXD(RS232_DCE_TXD),
    .APP_ADDR(1'h0),
    .APP_DATAIN(8'h0),
    .APP_DATAOUT(),
    .APP_WE(1'b0),
    .APP_RD(1'b0),
    .SPI_MISO(spi_miso_i),
    .SPI_MOSI(spi_mosi_i),
    .SPI_SCK(spi_sck_i),
    .SPI_SS_B(spi_ss_b_i),
    .PROG_B(),
    .DEBUG_RESET(debug_reset)
  );

  assign SPI_WP = 1'b1;

  generate
    if (SPI_DEVICE == "3S50AN"  || SPI_DEVICE == "3S200AN" ||
        SPI_DEVICE == "3S400AN" || SPI_DEVICE == "3S700AN" ||
        SPI_DEVICE == "3S1400AN")
      begin
        assign SPI_MOSI  = 1'b1;
        assign SPI_SCK   = 1'b1;
        assign SPI_SS_B   = 1'b1;
        assign spi_miso_i = 1'b1;
      end
    else
      begin
        assign SPI_MOSI  = spi_mosi_i;
        assign SPI_SCK   = spi_sck_i;
        assign SPI_SS_B   = spi_ss_b_i;
        assign spi_miso_i = SPI_MISO; 
      end

  endgenerate

endmodule
