//////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2006 Xilinx, Inc.
// This design is confidential and proprietary of Xilinx, All Rights Reserved.
//////////////////////////////////////////////////////////////////////////////
//   ____  ____
//  /   /\/   /
// /___/  \  /   Vendor:        Xilinx
// \   \   \/    Version:       2.0.0
//  \   \        Filename:      bootstrap.v
//  /   /        Date Created:  October 20, 2006
// /___/   /\    Last Modified: June 10, 2009
// \   \  /  \
//  \___\/\___\
//
// Devices:   Spartan-3 Generation FPGA
// Purpose:   Failsafe multiboot  
// Contact:   jimwe@xilinx.com
// Reference: None
//
// Revision History:
//   Rev 1.0.0 - (jimwe) First created October 20, 2006.
//                       Reuse of crabill's Spartan3A boarddemo
//   Rev 2.0.0 - (jimwe) Modified to support internal Spartan-3AN flash
//   Rev 2.1.0 - (jimwe) Removed dce_tx_pres signal which is no longer
//                       in the picoBlaze uart source
//
//////////////////////////////////////////////////////////////////////////////
//
// LIMITED WARRANTY AND DISCLAIMER. These designs are provided to you "as is".
// Xilinx and its licensors make and you receive no warranties or conditions,
// express, implied, statutory or otherwise, and Xilinx specifically disclaims
// any implied warranties of merchantability, non-infringement, or fitness for
// a particular purpose. Xilinx does not warrant that the functions contained
// in these designs will meet your requirements, or that the operation of
// these designs will be uninterrupted or error free, or that defects in the
// designs will be corrected. Furthermore, Xilinx does not warrant or make any
// representations regarding use or the results of the use of the designs in
// terms of correctness, accuracy, reliability, or otherwise.
//
// LIMITATION OF LIABILITY. In no event will Xilinx or its licensors be liable
// for any loss of data, lost profits, cost or procurement of substitute goods
// or services, or for any special, incidental, consequential, or indirect
// damages arising from the use or operation of the designs or accompanying
// documentation, however caused and on any theory of liability. This
// limitation will apply even if Xilinx has been advised of the possibility
// of such damage. This limitation shall apply not-withstanding the failure
// of the essential purpose of any limited remedies herein.
//
//////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2006 Xilinx, Inc.
// This design is confidential and proprietary of Xilinx, All Rights Reserved.
//////////////////////////////////////////////////////////////////////////////

`timescale 1 ns / 1 ps

module picoController
  #(
  parameter          SPI_DEVICE       = "3S700AN",
  parameter          USE_CRC          = "TRUE",
  parameter          USE_REG          = "TRUE",
  parameter          USE_JTAG         = "TRUE",
  parameter          USE_RS232        = "TRUE",
  parameter          USE_UNIQUE_CFG   = "FALSE",
  parameter          CHECK_CRC        = "TRUE",
  parameter          BOOTSTRAP_FW     = "TRUE"
  )
  (
  input  wire        CLK,
  input  wire        RESET,
  input  wire        PAUSE,
  input  wire [15:0] FPGA_REVISION,

  input  wire        RS232_DCE_RXD,
  output wire        RS232_DCE_TXD,

  input  wire        APP_ADDR,
  input  wire [7:0]  APP_DATAIN,
  output wire [7:0]  APP_DATAOUT,
  input  wire        APP_WE,
  input  wire        APP_RD,

 
  input  wire        SPI_MISO,
  output wire        SPI_MOSI,
  output wire        SPI_SCK,
  output wire        SPI_SS_B,

  output reg         PROG_B = 1,

  output wire        DEBUG_RESET

  );

  //******************************************************************//
  // Alternate locations for bitstreams and the history record can    //
  // be defined if need be by setting USE_UNIQUE_CFG = "TRUE" and     //
  // modifying those parameters                                       //
  //******************************************************************//

  wire    [7:0] port_id;
  wire    [7:0] NUMBER_OF_BITSTREAMS;
  wire   [23:0] BITSTREAM_OFFSET_1;
  wire   [23:0] BITSTREAM_OFFSET_2;
  wire   [23:0] BITSTREAM_OFFSET_3;
  wire   [23:0] HISTORY_OFFSET;
  reg     [7:0] alternate_config;

  generate 
    if (USE_UNIQUE_CFG == "TRUE")
      begin
        assign NUMBER_OF_BITSTREAMS = 3;
        assign BITSTREAM_OFFSET_1   = 24'h080000;
        assign BITSTREAM_OFFSET_2   = 24'h100000;
        assign BITSTREAM_OFFSET_3   = 24'h180000;
        assign HISTORY_OFFSET       = 24'h070000; 
      end
    else if (SPI_DEVICE == "M25P16")
      begin
        assign NUMBER_OF_BITSTREAMS = 3;
        assign BITSTREAM_OFFSET_1   = 24'h080000;
        assign BITSTREAM_OFFSET_2   = 24'h100000;
        assign BITSTREAM_OFFSET_3   = 24'h180000;
        assign HISTORY_OFFSET       = 24'h070000;
      end
    else if (SPI_DEVICE == "AT45DB161D")
      begin
        assign NUMBER_OF_BITSTREAMS = 3;
        assign BITSTREAM_OFFSET_1   = 24'h100000;
        assign BITSTREAM_OFFSET_2   = 24'h200000;
        assign BITSTREAM_OFFSET_3   = 24'h300000;
        assign HISTORY_OFFSET       = 24'h3FFC00;
      end
    else if (SPI_DEVICE == "3S50AN")
      begin
        assign NUMBER_OF_BITSTREAMS = 1;
        assign BITSTREAM_OFFSET_1   = 24'h020000;
        assign BITSTREAM_OFFSET_2   = 24'h000000;
        assign BITSTREAM_OFFSET_3   = 24'h000000;
        assign HISTORY_OFFSET       = 24'h03FE00;
      end
    else if (SPI_DEVICE == "3S200AN")
      begin
        assign NUMBER_OF_BITSTREAMS = 1;
        assign BITSTREAM_OFFSET_1   = 24'h080000;
        assign BITSTREAM_OFFSET_2   = 24'h000000;
        assign BITSTREAM_OFFSET_3   = 24'h000000;
        assign HISTORY_OFFSET       = 24'h0FFE00;
      end
    else if (SPI_DEVICE == "3S400AN")
      begin
        assign NUMBER_OF_BITSTREAMS = 1;
        assign BITSTREAM_OFFSET_1   = 24'h080000;
        assign BITSTREAM_OFFSET_2   = 24'h000000;
        assign BITSTREAM_OFFSET_3   = 24'h000000;
        assign HISTORY_OFFSET       = 24'h0FFE00;
      end
    else if (SPI_DEVICE == "3S700AN")
      begin
        assign NUMBER_OF_BITSTREAMS = 1;
        assign BITSTREAM_OFFSET_1   = 24'h100000;
        assign BITSTREAM_OFFSET_2   = 24'h000000;
        assign BITSTREAM_OFFSET_3   = 24'h000000;
        assign HISTORY_OFFSET       = 24'h1FFE00;
      end
    else if (SPI_DEVICE == "3S1400AN")
      begin
        assign NUMBER_OF_BITSTREAMS = 1;
        assign BITSTREAM_OFFSET_1   = 24'h200000;
        assign BITSTREAM_OFFSET_2   = 24'h000000;
        assign BITSTREAM_OFFSET_3   = 24'h000000;
        assign HISTORY_OFFSET       = 24'h3FFC00;
      end
  endgenerate    


  always @(port_id)
  begin
    case(port_id[3:0])
      4'h0 : alternate_config = NUMBER_OF_BITSTREAMS;
      4'h1 : alternate_config = BITSTREAM_OFFSET_1[23:16];
      4'h2 : alternate_config = BITSTREAM_OFFSET_2[23:16];
      4'h3 : alternate_config = BITSTREAM_OFFSET_3[23:16];
      4'h6 : alternate_config = HISTORY_OFFSET[23:16];
      4'h7 : alternate_config = HISTORY_OFFSET[15:8];
      default : alternate_config = 8'h00;
    endcase
  end

  //******************************************************************//
  // The picoblaze reset is the logical OR of an external reset or    // 
  // a register command based reset                                   //
  //******************************************************************//
    
  wire   reset_i;
  wire   reset_command;

  assign reset_i = RESET || reset_command;

  //******************************************************************//
  // Instantiate SPI_ACCESS for internal FLASH (if necessary)         // 
  // otherwise connect the spi signals to the output ports            //
  //******************************************************************//

  wire          spi_miso_i;
  wire          spi_mosi_i;
  wire          spi_sck_i;
  wire          spi_ss_b_i;
  wire          internal_flash;

  generate 
    if (SPI_DEVICE == "3S50AN"  || SPI_DEVICE == "3S200AN" ||
        SPI_DEVICE == "3S400AN" || SPI_DEVICE == "3S700AN" ||
        SPI_DEVICE == "3S1400AN" )
      begin
        assign SPI_MOSI  = 1'b1;
        assign SPI_SCK   = 1'b1;
        assign SPI_SS_B  = 1'b1;
        assign internal_flash = 1'b1;

        SPI_ACCESS #(.SIM_DEVICE(SPI_DEVICE))
        spiAccess_inst (
          .MOSI(spi_mosi_i),
          .MISO(spi_miso_i),
          .CSB(spi_ss_b_i),
          .CLK(spi_sck_i)
        );     
      end
   else 
      begin
        assign SPI_MOSI  = spi_mosi_i;
        assign SPI_SCK   = spi_sck_i;
        assign SPI_SS_B   = spi_ss_b_i;
        assign spi_miso_i = SPI_MISO;
        assign internal_flash = 1'b0;
      end
  endgenerate

  //******************************************************************//
  // Instantiate PicoBlaze and the Program ROM.                       // 
  //******************************************************************//

  wire    [9:0] address;
  wire   [17:0] instruction;
  wire    [7:0] out_port;
  reg     [7:0] in_port;
  wire          write_strobe;
  wire          read_strobe;
  wire          interrupt;

  kcpsm3 kcpsm3_inst (
    .address(address),
    .instruction(instruction),
    .port_id(port_id),
    .write_strobe(write_strobe),
    .out_port(out_port),
    .read_strobe(read_strobe),
    .in_port(in_port),
    .interrupt(interrupt),
    .interrupt_ack(),
    .reset(reset_i),
    .clk(CLK));

  picocode picocode_inst (
    .address(address),
    .instruction(instruction),
    .debug_reset(DEBUG_RESET),
    .clk(CLK));


  //******************************************************************//
  // Implement Register interface                                     //
  // 00h rd = rx data                                                 //
  // 00h wr = tx data                                                 //
  // 01h rd = status                                                  //
  //          bit 7 -- tx buffer data present                         //
  //          bit 4 -- tx buffer full                                 //
  //          bit 3 -- rx buffer full                                 //
  //          bit 0 -- rx buffer data present                         //
  //          others not implemented                                  //
  // 01h wr = not implemented                                         //
  //******************************************************************//

  wire          reg_wr;
  reg           reg_rd;
  wire          reg_tx_full;
  wire          reg_tx_pres;
  wire          reg_rx_full;
  wire          reg_rx_pres;
  wire    [7:0] reg_rx_data;

  generate
    if (USE_REG == "TRUE") 
      begin

        always @(posedge CLK)
        begin
          reg_rd <= read_strobe && (port_id == 8'h02);
        end
      
        assign reg_wr = write_strobe && (port_id == 8'h02);
      
        picoRegs regInst (
          .clk(CLK),
          .app_addr(APP_ADDR),
          .app_datain(APP_DATAIN),
          .app_dataout(APP_DATAOUT),
          .app_rd(APP_RD),
          .app_we(APP_WE),
          .reset_buffer(reset_i),
          .read_buffer(reg_rd),
          .write_buffer(reg_wr),
          .data_in(out_port), 
          .tbuffer_data_present(reg_tx_pres),
          .tbuffer_half_full(),
          .tbuffer_full(reg_tx_full),
          .data_out(reg_rx_data),
          .rbuffer_data_present(reg_rx_pres),
          .rbuffer_half_full(),
          .rbuffer_full(reg_rx_full));
      end
    else
      begin
        assign reg_tx_full = 1'b0;
        assign reg_tx_pres = 1'b0;
        assign reg_rx_full = 1'b0;
        assign reg_rx_pres = 1'b0;
        assign reg_rx_data = 8'h00;
      end
  endgenerate

  //******************************************************************//
  // Implement JTAG interface                                         //
  // 10h rd = rx data                                                 //
  // 10h wr = tx data                                                 //
  // 11h rd = status                                                  //
  //          bit 7 -- tx buffer data present                         //
  //          bit 4 -- tx buffer full                                 //
  //          bit 3 -- rx buffer full                                 //
  //          bit 0 -- rx buffer data present                         //
  //          others not implemented                                  //
  // 11h wr = not implemented                                         //
  //******************************************************************//

  wire          jtag_wr;
  reg           jtag_rd;
  wire          jtag_tx_full;
  wire          jtag_tx_pres;
  wire          jtag_rx_full;
  wire          jtag_rx_pres;
  wire    [7:0] jtag_rx_data;

  generate
    if (USE_JTAG == "TRUE") 
      begin

        always @(posedge CLK)
        begin
          jtag_rd <= read_strobe && (port_id == 8'h10);
        end
      
        assign jtag_wr = write_strobe && (port_id == 8'h10);
      
        picoJtag jtagInst (
          .clk(CLK),
          .reset_buffer(reset_i),
          .read_buffer(jtag_rd),
          .write_buffer(jtag_wr),
          .data_in(out_port), 
          .tbuffer_data_present(jtag_tx_pres),
          .tbuffer_half_full(),
          .tbuffer_full(jtag_tx_full),
          .data_out(jtag_rx_data),
          .rbuffer_data_present(jtag_rx_pres),
          .rbuffer_half_full(),
          .rbuffer_full(jtag_rx_full));
      end
    else
      begin
        assign jtag_tx_full = 1'b0;
        assign jtag_tx_pres = 1'b0;
        assign jtag_rx_full = 1'b0;
        assign jtag_rx_pres = 1'b0;
        assign jtag_rx_data = 8'h00;
      end
  endgenerate

 
  //******************************************************************//
  // Implement DCE serial port, 115200-8N1                            //
  // 20h rd = rx data                                                 //
  // 20h wr = tx data                                                 //
  // 21h rd = status                                                  //
  //          bit 7 -- tx buffer data present                         //
  //          bit 4 -- tx buffer full                                 //
  //          bit 3 -- rx buffer full                                 //
  //          bit 0 -- rx buffer data present                         //
  //          others not implemented                                  //
  // 21h wr = not implemented                                         //
  //******************************************************************//

  reg     [7:0] en_16_x_cntr;
  wire          en_16_x_baud;
  wire          dce_wr;
  reg           dce_rd;
  wire          dce_tx_full;
  wire          dce_rx_full;
  wire          dce_rx_pres;
  wire    [7:0] dce_rx_data;

  generate
    if (USE_RS232 == "TRUE") 
      begin

        always @(posedge CLK)
        begin
          if (en_16_x_baud) en_16_x_cntr <= 0;
          else en_16_x_cntr <= en_16_x_cntr + 1;
          dce_rd <= read_strobe && (port_id == 8'h20);
        end
      
        assign en_16_x_baud = (en_16_x_cntr == 13);
        assign dce_wr = write_strobe && (port_id == 8'h20);
      
        uart_tx uart_tx_dce (
          .en_16_x_baud(en_16_x_baud),
          .serial_out(RS232_DCE_TXD),
          .reset_buffer(reset_i),
          .write_buffer(dce_wr),
          .data_in(out_port), 
          .buffer_half_full(),
          .buffer_full(dce_tx_full),
          .clk(CLK));
      
        uart_rx uart_rx_dce (
          .en_16_x_baud(en_16_x_baud),
          .serial_in(RS232_DCE_RXD),
          .reset_buffer(reset_i),
          .read_buffer(dce_rd),
          .data_out(dce_rx_data),
          .buffer_data_present(dce_rx_pres),
          .buffer_half_full(),
          .buffer_full(dce_rx_full),
          .clk(CLK));  
      end
    else
      begin
        assign dce_tx_full = 1'b0;
        assign dce_rx_full = 1'b0;
        assign dce_rx_pres = 1'b0;
        assign dce_rx_data = 8'h00;
      end
  endgenerate

  //******************************************************************//
  // Implement SPI port                                               //
  //******************************************************************//

  wire          spiReg_rd;
  wire          spiReg_we;
  wire   [7:0]  spiReg_dataout;

  assign spiReg_rd = read_strobe  & (port_id[7:4] == 4'h3);
  assign spiReg_we = write_strobe & (port_id[7:4] == 4'h3);

  spi spi_Inst(
    .CLK(CLK),
    .RESET(reset_i),
    .PICO_ADDR(port_id[2:0]),
    .PICO_DATAIN(out_port),
    .PICO_DATAOUT(spiReg_dataout),
    .PICO_WE(spiReg_we),
    .PICO_RD(spiReg_rd),
    .miso(spi_miso_i),
    .mosi(spi_mosi_i),
    .sck(spi_sck_i),
    .ss_b(spi_ss_b_i)
  );
  
  //******************************************************************//
  // Instantiate the multiboot module.  Of course, multiboot          //
  // could have been implemented completely in software, but this     //
  // provided the opportunity to build a hardware module that could   //
  // be used in other designs -- with or without processors.          //
  // 40h rd = mb_int_addr[ 7: 0]                                      //
  // 40h wr = mb_int_addr[ 7: 0]                                      //
  // 41h rd = mb_int_addr[15: 8]                                      //
  // 41h wr = mb_int_addr[15: 8]                                      //
  // 42h rd = mb_int_addr[23:16]                                      //
  // 42h wr = mb_int_addr[23:16]                                      //
  // 43h rd = mb_int_addr[31:24]                                      //
  // 43h wr = mb_int_addr[31:24]                                      //
  // 44h rd = mb_int_mode[2:0]                                        //
  // 44h wr = mb_int_mode[2:0]                                        //
  // 45h rd = mb_int_vsel[2:0]                                        //
  // 45h wr = mb_int_vsel[2:0]                                        //
  // 46h rd = mb_int_use                                              //
  // 46h wr = mb_int_use                                              //
  // 47h rd = unused                                                  //
  // 47h wr = multiboot strobe                                        //
  //******************************************************************//

  reg    [31:0] mb_int_addr;
  reg     [2:0] mb_int_mode;
  reg     [2:0] mb_int_vsel;
  reg           mb_int_use;
  wire          reboot;

  always @(posedge CLK)
  begin
    if (write_strobe)
    begin
      if (port_id == 8'h40) mb_int_addr[7:0] <= out_port;
      if (port_id == 8'h41) mb_int_addr[15:8] <= out_port;
      if (port_id == 8'h42) mb_int_addr[23:16] <= out_port;
      if (port_id == 8'h43) mb_int_addr[31:24] <= out_port;
      if (port_id == 8'h44) mb_int_mode <= out_port[2:0];
      if (port_id == 8'h45) mb_int_vsel <= out_port[2:0];
      if (port_id == 8'h46) mb_int_use <= out_port[0];
    end
  end

  assign reboot = write_strobe && (port_id == 8'h47);

  picoboot picoboot_inst (
    .internal_addr(mb_int_addr),
    .internal_mode(mb_int_mode),
    .internal_vsel(mb_int_vsel),
    .internal_use(mb_int_use),
    .reboot(reboot),
    .clk(CLK)
  );

   //******************************************************************//
  // Implement CRC16
  //******************************************************************//

  wire   [15:0] crc;
  wire          resetCrc;
  wire          useData;

  assign resetCrc = write_strobe & (port_id[7:0] == 8'h60);
  assign useData  = write_strobe & (port_id[7:0] == 8'h61);

  generate
    if (USE_CRC == "TRUE") 
      begin
        picoCrc picoCrc_Inst(
          .clk(CLK),
          .resetCrc(resetCrc),
          .useData(useData),
          .Din(out_port),
          .Cout(crc) 
        );
      end
    else
      begin
        assign crc = 8'h00;
      end
  endgenerate

  //******************************************************************//
  // Reset command
  //******************************************************************//

    assign reset_command = write_strobe && (port_id == 8'hFF);

  //******************************************************************//
  // PROG_B command
  //******************************************************************//
  
  always @(posedge CLK)
  begin
    if (write_strobe)
    begin
      if (port_id == 8'h50) PROG_B <= ~(out_port == 8'hFF);
    end
  end

  //******************************************************************//
  // Implement input port muxing logic.                               //
  //******************************************************************//

  wire [7:0] firmware_config;

  assign firmware_config[7:4] = 4'h0;
  assign firmware_config[2]   = internal_flash;
  assign firmware_config[1]   = PAUSE;

  generate 
    if (CHECK_CRC == "TRUE")
      assign firmware_config[3] = 1'b1;
    else
      assign firmware_config[3] = 1'b0;
  endgenerate

  generate 
    if (BOOTSTRAP_FW == "TRUE")
      assign firmware_config[0] = 1'b1;
    else
      assign firmware_config[0] = 1'b0;
  endgenerate

  always @(*)
  begin
    casex (port_id)
      8'h00: in_port <= FPGA_REVISION[7:0];
      8'h01: in_port <= FPGA_REVISION[15:8];
      8'h02: in_port <= reg_rx_data;
      8'h03: in_port <= {reg_tx_pres,2'b00,reg_tx_full,
                         reg_rx_full,2'b00,reg_rx_pres};
      8'h10: in_port <= jtag_rx_data;
      8'h11: in_port <= {jtag_tx_pres,2'b00,jtag_tx_full,
                         jtag_rx_full,2'b00,jtag_rx_pres};
      8'h20: in_port <= dce_rx_data;
      8'h21: in_port <= {3'b000,dce_tx_full,
                         dce_rx_full,2'b00,dce_rx_pres};
      8'h3x: in_port <= spiReg_dataout;
      8'h40: in_port <= mb_int_addr[7:0];
      8'h41: in_port <= mb_int_addr[15:8];
      8'h42: in_port <= mb_int_addr[23:16];
      8'h43: in_port <= mb_int_addr[31:24];
      8'h44: in_port <= {5'b00000,mb_int_mode};
      8'h45: in_port <= {5'b00000,mb_int_vsel};
      8'h46: in_port <= {7'b0000000,mb_int_use};
      8'h50: in_port <= firmware_config;
      8'h61: in_port <= crc[7:0];
      8'h62: in_port <= crc[15:8];
      8'h7x: in_port <= alternate_config;
      default: in_port <= 8'h00;
    endcase
  end


endmodule
