------------------------------------------------------------------------------
-- Copyright (c) 2008 Xilinx, Inc.
-- This design is confidential and proprietary of Xilinx, All Rights Reserved.
------------------------------------------------------------------------------
--   ____  ____
--  /   /\/   /
-- /___/  \  /   Vendor:        Xilinx
-- \   \   \/    Version:       1.0.0
--  \   \        Filename:      application1.vhd
--  /   /        Date Created:  April 28, 2008
-- /___/   /\    Last Modified: April 28, 2008
-- \   \  /  \
--  \___\/\___\
--
-- Devices:   Spartan-3 Generation FPGA
-- Purpose:   Failsafe multiboot  
-- Contact:   jimwe@xilinx.com
-- Reference: None
--
-- Revision History:
--   Rev 1.0.0 - (jimwe) First created October 20, 2006.
--                       Reuse of crabill's Spartan3A boarddemo
--
--   Rev 1.1.0 - (jimwe) Added support for Spartan 3AN
--
------------------------------------------------------------------------------
--
-- LIMITED WARRANTY AND DISCLAIMER. These designs are provided to you "as is".
-- Xilinx and its licensors make and you receive no warranties or conditions,
-- express, implied, statutory or otherwise, and Xilinx specifically disclaims
-- any implied warranties of merchantability, non-infringement, or fitness for
-- a particular purpose. Xilinx does not warrant that the functions contained
-- in these designs will meet your requirements, or that the operation of
-- these designs will be uninterrupted or error free, or that defects in the
-- designs will be corrected. Furthermore, Xilinx does not warrant or make any
-- representations regarding use or the results of the use of the designs in
-- terms of correctness, accuracy, reliability, or otherwise.
--
-- LIMITATION OF LIABILITY. In no event will Xilinx or its licensors be liable
-- for any loss of data, lost profits, cost or procurement of substitute goods
-- or services, or for any special, incidental, consequential, or indirect
-- damages arising from the use or operation of the designs or accompanying
-- documentation, however caused and on any theory of liability. This
-- limitation will apply even if Xilinx has been advised of the possibility
-- of such damage. This limitation shall apply not-withstanding the failure
-- of the essential purpose of any limited remedies herein.
--
------------------------------------------------------------------------------
-- Copyright (c) 2008 Xilinx, Inc.
-- This design is confidential and proprietary of Xilinx, All Rights Reserved.
------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_UNSIGNED.all;
use IEEE.STD_LOGIC_MISC.all;

library UNISIM;
use UNISIM.VCOMPONENTS.all;

entity APPLICATION is
  port (
    CLK_50MHZ     : in  std_logic;
    RESET         : in  std_logic;
    PAUSE         : in  std_logic;
    RS232_DCE_RXD : in  std_logic;
    RS232_DCE_TXD : out std_logic;
    SPI_MISO      : in  std_logic;
    SPI_MOSI      : out std_logic;
    SPI_SCK       : out std_logic;
    SPI_SS_B      : out std_logic;
    SPI_WP        : out std_logic;
    LD            : out std_logic_vector(7 downto 0) );
end APPLICATION;

architecture RTL of APPLICATION is

  component SYNCHRO
    generic (
      INITIALIZE : string := "LOGIC0");
    port (
      ASYNC : in  std_logic;
      CLK   : in  std_logic;
      SYNC  : out std_logic);
  end component;

  component PICOCONTROLLER
    generic (
      SPI_DEVICE     : string := "3S700AN";
      USE_CRC        : string := "TRUE";
      USE_REG        : string := "TRUE";
      USE_JTAG       : string := "TRUE";
      USE_RS232      : string := "TRUE";
      USE_UNIQUE_CFG : string := "FALSE";
      CHECK_CRC      : string := "TRUE";
      BOOTSTRAP_FW   : string := "TRUE" );
    port (
      CLK            : in  std_logic;
      RESET          : in  std_logic;
      PAUSE          : in  std_logic;
      FPGA_REVISION  : in  std_logic_vector(15 downto 0);
      RS232_DCE_RXD  : in  std_logic;
      RS232_DCE_TXD  : out std_logic;
      APP_ADDR       : in  std_logic;
      APP_DATAIN     : in  std_logic_vector(7 downto 0);  
      APP_DATAOUT    : out std_logic_vector(7 downto 0);
      APP_WE         : in  std_logic;
      APP_RD         : in  std_logic;
      SPI_MISO       : in  std_logic;  
      SPI_MOSI       : out std_logic;  
      SPI_SCK        : out std_logic;  
      SPI_SS_B       : out std_logic;  
      PROG_B         : out std_logic := '1';
      DEBUG_RESET    : out std_logic );
  end component;

  constant SPI_DEVICE : string := "AT45DB161D";
  constant USE_REG    : string := "TRUE";
  constant USE_JTAG   : string := "FALSE";
  constant USE_RS232  : string := "TRUE";
  constant CHECK_CRC  : string := "TRUE";
  constant BOOTSTRAP_FW : string := "FALSE";

  signal clk_25mhz    : std_logic;
  signal clknub       : std_logic;
  signal locked       : std_logic;
  signal reset_i      : std_logic;
  signal debug_reset  : std_logic;
  signal combo_reset  : std_logic;
  signal clk_divide   : std_logic_vector(20 downto 0);
  signal clk_enable   : std_logic;
  signal spi_miso_i   : std_logic;
  signal spi_mosi_i   : std_logic;
  signal spi_sck_i    : std_logic;
  signal spi_ss_b_i   : std_logic;
  signal ld_i         : std_logic_vector(7 downto 0);

begin

  --******************************************************************//
  -- Create global clock and asynchronous system reset.               //
  --******************************************************************//

  DCM_SP_INST : DCM_SP
    generic map (
      CLKIN_DIVIDE_BY_2 => TRUE,
      CLKIN_PERIOD      => 20.000 )
    port map (
      CLKIN    => CLK_50MHZ,
      CLKFB    => clk_25mhz,
      RST      => '0',
      PSEN     => '0',
      PSINCDEC => '0',
      PSCLK    => '0',
      DSSEN    => '0',
      CLK0     => clknub,
      CLK90    => open,
      CLK180   => open,
      CLK270   => open,
      CLKDV    => open,
      CLK2X    => open,
      CLK2X180 => open,
      CLKFX    => open,
      CLKFX180 => open,
      STATUS   => open,
      LOCKED   => locked,
      PSDONE   => open );

  BG_0 : BUFG 
    port map (
      I => clknub,
      O => clk_25mhz );

  combo_reset <= (not locked) or debug_reset or RESET;

  synchro_reset : synchro 
    generic map (
      INITIALIZE => "LOGIC1" )
    port map (
      async => combo_reset,
      sync  => reset_i,
      clk   => clk_25mhz );
 
  --******************************************************************//
  -- Instantiate an i'm alive LED
  --******************************************************************//

  clkdiv_process : process(clk_25mhz)
  begin
    if rising_edge(clk_25mhz) then
      if (reset_i = '1') then
        clk_enable <= '0';
        clk_divide <= (others => '0');
      else
        clk_enable <= and_reduce(clk_divide);
        clk_divide <= clk_divide + '1';
      end if;
    end if;
  end process clkdiv_process;

  led_process : process(clk_25mhz)
  begin
    if rising_edge(clk_25mhz) then
      if (reset_i = '1') then
        ld_i <= X"01";
      elsif (clk_enable = '1') then
        ld_i <= ld_i(6 downto 0) & ld_i(7);
      end if;
    end if;
  end process led_process;

  LD <= ld_i;

  --******************************************************************//
  -- Instantiate picoController
  --******************************************************************//

  picoController_inst : picoController 
    generic map (
      SPI_DEVICE    => SPI_DEVICE,
      USE_REG       => USE_REG,
      USE_JTAG      => USE_JTAG,
      USE_RS232     => USE_RS232,
      CHECK_CRC     => CHECK_CRC,
      BOOTSTRAP_FW  => BOOTSTRAP_FW )
    port map (
      CLK           => clk_25mhz,
      RESET         => reset_i,
      PAUSE         => PAUSE,
      FPGA_REVISION => X"0102",
      RS232_DCE_RXD => RS232_DCE_RXD,
      RS232_DCE_TXD => RS232_DCE_TXD,
      APP_ADDR      => '0',
      APP_DATAIN    => X"00",
      APP_DATAOUT   => open,
      APP_WE        => '0',
      APP_RD        => '0',
      SPI_MISO      => spi_miso_i,
      SPI_MOSI      => spi_mosi_i,
      SPI_SCK       => spi_sck_i,
      SPI_SS_B      => spi_ss_b_i,
      PROG_B        => open,
      DEBUG_RESET   => debug_reset );
  
  SPI_WP <= '1';

  spi_static_map : if (SPI_DEVICE = "3S50AN"  or SPI_DEVICE = "3S200AN" or
                       SPI_DEVICE = "3S400AN" or SPI_DEVICE = "3S700AN" or
                       SPI_DEVICE = "3S1400AN") generate
  begin
     SPI_MOSI <= '1';
     SPI_SCK  <= '1';
     SPI_SS_B <= '1';
     spi_miso_i <= '1';
  end generate spi_static_map;

  spi_signal_map : if (not(SPI_DEVICE = "3S50AN"  or SPI_DEVICE = "3S200AN" or
                           SPI_DEVICE = "3S400AN" or SPI_DEVICE = "3S700AN" or
                           SPI_DEVICE = "3S1400AN")) generate
  begin
    SPI_MOSI <= spi_mosi_i;
    SPI_SCK  <= spi_sck_i;
    SPI_SS_B <= spi_ss_b_i;
    spi_miso_i <= SPI_MISO;
  end generate spi_signal_map;

end RTL;
