------------------------------------------------------------------------------
-- Copyright (c) 2008 Xilinx, Inc.
-- This design is confidential and proprietary of Xilinx, All Rights Reserved.
------------------------------------------------------------------------------
--   ____  ____
--  /   /\/   /
-- /___/  \  /   Vendor:        Xilinx
-- \   \   \/    Version:       2.0.0
--  \   \        Filename:      bootstrap.v
--  /   /        Date Created:  April 25, 2008
-- /___/   /\    Last Modified: June 10, 2008
-- \   \  /  \
--  \___\/\___\
--
-- Devices:   Spartan-3 Generation FPGA
-- Purpose:   Failsafe multiboot  
-- Contact:   jimwe@xilinx.com
-- Reference: None
--
-- Revision History:
--   Rev 1.0.0 - (jimwe) Converted to VHDL from verilog
--   Rev 1.1.0 - (jimwe) Removed the dce_tx_pres port which is no longer
--                       in the picoBlaze source code
--
------------------------------------------------------------------------------
--
-- LIMITED WARRANTY AND DISCLAIMER. These designs are provided to you "as is".
-- Xilinx and its licensors make and you receive no warranties or conditions,
-- express, implied, statutory or otherwise, and Xilinx specifically disclaims
-- any implied warranties of merchantability, non-infringement, or fitness for
-- a particular purpose. Xilinx does not warrant that the functions contained
-- in these designs will meet your requirements, or that the operation of
-- these designs will be uninterrupted or error free, or that defects in the
-- designs will be corrected. Furthermore, Xilinx does not warrant or make any
-- representations regarding use or the results of the use of the designs in
-- terms of correctness, accuracy, reliability, or otherwise.
--
-- LIMITATION OF LIABILITY. In no event will Xilinx or its licensors be liable
-- for any loss of data, lost profits, cost or procurement of substitute goods
-- or services, or for any special, incidental, consequential, or indirect
-- damages arising from the use or operation of the designs or accompanying
-- documentation, however caused and on any theory of liability. This
-- limitation will apply even if Xilinx has been advised of the possibility
-- of such damage. This limitation shall apply not-withstanding the failure
-- of the essential purpose of any limited remedies herein.
--
------------------------------------------------------------------------------
-- Copyright (c) 2006 Xilinx, Inc.
-- This design is confidential and proprietary of Xilinx, All Rights Reserved.
------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

library UNISIM;
use UNISIM.VCOMPONENTS.all;

entity PICOCONTROLLER is 
  generic (
    SPI_DEVICE     : string := "3S700AN";
    USE_CRC        : string := "TRUE";
    USE_REG        : string := "TRUE";
    USE_JTAG       : string := "TRUE";
    USE_RS232      : string := "TRUE";
    USE_UNIQUE_CFG : string := "FALSE";
    CHECK_CRC      : string := "TRUE";
    BOOTSTRAP_FW   : string := "TRUE" );
  port (
    CLK            : in  std_logic;
    RESET          : in  std_logic;
    PAUSE          : in  std_logic;
    FPGA_REVISION  : in  std_logic_vector(15 downto 0);
    RS232_DCE_RXD  : in  std_logic;
    RS232_DCE_TXD  : out std_logic;
    APP_ADDR       : in  std_logic;
    APP_DATAIN     : in  std_logic_vector(7 downto 0);  
    APP_DATAOUT    : out std_logic_vector(7 downto 0);
    APP_WE         : in  std_logic;
    APP_RD         : in  std_logic;
    SPI_MISO       : in  std_logic;  
    SPI_MOSI       : out std_logic;  
    SPI_SCK        : out std_logic;  
    SPI_SS_B       : out std_logic;  
    PROG_B         : out std_logic := '1';
    DEBUG_RESET    : out std_logic );
end PICOCONTROLLER;

architecture RTL of PICOCONTROLLER is 

  component KCPSM3 
    port (     
      address : out std_logic_vector(9 downto 0);
      instruction : in std_logic_vector(17 downto 0);
      port_id : out std_logic_vector(7 downto 0);
      write_strobe : out std_logic;
      out_port : out std_logic_vector(7 downto 0);
      read_strobe : out std_logic;
      in_port : in std_logic_vector(7 downto 0);
      interrupt : in std_logic;
      interrupt_ack : out std_logic;
      reset : in std_logic;
      clk : in std_logic);
  end component;

  component PICOCODE
    port (
      address : in std_logic_vector(9 downto 0);
      instruction : out std_logic_vector(17 downto 0);
      clk : in std_logic);
  end component;

  component PICOBOOT is
    port (
      INTERNAL_ADDR : in  std_logic_vector(31 downto 0);
      INTERNAL_MODE : in  std_logic_vector(2 downto 0);
      INTERNAL_VSEL : in  std_logic_vector(2 downto 0);
      INTERNAL_USE  : in  std_logic;
      REBOOT        : in  std_logic;
      CLK           : in  std_logic);
  end component;

  component PICOCRC
    port (
      CLK           : in  std_logic;
      RESETCRC      : in  std_logic;
      USEDATA       : in  std_logic;
      DIN           : in  std_logic_vector(7 downto 0);
      COUT          : out std_logic_vector(15 downto 0));
  end component;

  component PICOJTAG
    port (
      CLK           : in  std_logic;
      RESET_BUFFER  : in  std_logic;
      READ_BUFFER   : in  std_logic;
      WRITE_BUFFER  : in  std_logic;
      DATA_IN       : in  std_logic_vector(7 downto 0);
      DATA_OUT      : out std_logic_vector(7 downto 0);
      TBUFFER_DATA_PRESENT : out std_logic;
      TBUFFER_HALF_FULL    : out std_logic;
      TBUFFER_FULL         : out std_logic;
      RBUFFER_DATA_PRESENT : out std_logic;
      RBUFFER_HALF_FULL    : out std_logic;
      RBUFFER_FULL         : out std_logic );
  end component;

  component PICOREGS
    port (
      CLK           : in  std_logic;
      APP_ADDR      : in  std_logic;
      APP_DATAIN    : in  std_logic_vector(7 downto 0);
      APP_DATAOUT   : out std_logic_vector(7 downto 0);
      APP_RD        : in  std_logic;
      APP_WE        : in  std_logic;
      RESET_BUFFER  : in  std_logic;
      READ_BUFFER   : in  std_logic;
      WRITE_BUFFER  : in  std_logic;
      DATA_IN       : in  std_logic_vector(7 downto 0);
      DATA_OUT      : out std_logic_vector(7 downto 0);
      TBUFFER_DATA_PRESENT : out std_logic;
      TBUFFER_HALF_FULL    : out std_logic;
      TBUFFER_FULL         : out std_logic;
      RBUFFER_DATA_PRESENT : out std_logic;
      RBUFFER_HALF_FULL    : out std_logic;
      RBUFFER_FULL         : out std_logic );
  end component;

  component SPI
    port (
      CLK           : in  std_logic;
      RESET         : in  std_logic;
      PICO_ADDR     : in  std_logic_vector(2 downto 0);
      PICO_DATAIN   : in  std_logic_vector(7 downto 0);
      PICO_DATAOUT  : out std_logic_vector(7 downto 0);
      PICO_WE       : in  std_logic;
      PICO_RD       : in  std_logic;
      MISO          : in  std_logic;
      MOSI          : out std_logic;
      SCK           : out std_logic;
      SS_B          : out std_logic);
  end component;

  component UART_RX is
    port (
      serial_in : in std_logic;
      data_out : out std_logic_vector(7 downto 0);
      read_buffer : in std_logic;
      reset_buffer : in std_logic;
      en_16_x_baud : in std_logic;
      buffer_data_present : out std_logic;
      buffer_full : out std_logic;
      buffer_half_full : out std_logic;
      clk : in std_logic);
  end component;

  component UART_TX
    port (    
      data_in : in std_logic_vector(7 downto 0);
      write_buffer : in std_logic;
      reset_buffer : in std_logic;
      en_16_x_baud : in std_logic;
      serial_out : out std_logic;
      buffer_full : out std_logic;
      buffer_half_full : out std_logic;
      clk : in std_logic);
  end component;

  signal port_id              : std_logic_vector(7 downto 0);
  signal number_of_bitstreams : std_logic_vector(7 downto 0);
  signal bitstream_offset_1   : std_logic_vector(23 downto 0);
  signal bitstream_offset_2   : std_logic_vector(23 downto 0);
  signal bitstream_offset_3   : std_logic_vector(23 downto 0);
  signal history_offset       : std_logic_vector(23 downto 0);
  signal alternate_config     : std_logic_vector(7 downto 0);
  signal reset_i              : std_logic;
  signal reset_command        : std_logic;
  signal spi_mosi_i           : std_logic;
  signal spi_miso_i           : std_logic;
  signal spi_sck_i            : std_logic;
  signal spi_ss_b_i           : std_logic;
  signal internal_flash       : std_logic;
  signal address              : std_logic_vector(9 downto 0);
  signal instruction          : std_logic_vector(17 downto 0);
  signal out_port             : std_logic_vector(7 downto 0);
  signal in_port              : std_logic_vector(7 downto 0);
  signal write_strobe         : std_logic;
  signal read_strobe          : std_logic;
  signal interrupt            : std_logic;
  signal reg_wr               : std_logic;
  signal reg_rd               : std_logic;
  signal reg_tx_full          : std_logic;
  signal reg_tx_pres          : std_logic;
  signal reg_rx_full          : std_logic;
  signal reg_rx_pres          : std_logic;
  signal reg_rx_data          : std_logic_vector(7 downto 0);
  signal jtag_wr              : std_logic;
  signal jtag_rd              : std_logic;
  signal jtag_tx_full         : std_logic;
  signal jtag_tx_pres         : std_logic;
  signal jtag_rx_full         : std_logic;
  signal jtag_rx_pres         : std_logic;
  signal jtag_rx_data         : std_logic_vector(7 downto 0);
  signal en_16_x_cntr         : std_logic_vector(7 downto 0);
  signal en_16_x_baud         : std_logic;
  signal dce_wr               : std_logic;
  signal dce_rd               : std_logic;
  signal dce_tx_full          : std_logic;
  signal dce_rx_full          : std_logic;
  signal dce_rx_pres          : std_logic;
  signal dce_rx_data          : std_logic_vector(7 downto 0);
  signal spireg_rd            : std_logic;
  signal spireg_we            : std_logic;
  signal spireg_dataout       : std_logic_vector(7 downto 0);
  signal mb_int_addr          : std_logic_vector(31 downto 0);
  signal mb_int_mode          : std_logic_vector(2 downto 0);
  signal mb_int_vsel          : std_logic_vector(2 downto 0);
  signal mb_int_use           : std_logic;
  signal reboot               : std_logic;
  signal crc                  : std_logic_vector(15 downto 0);
  signal resetcrc             : std_logic;
  signal usedata              : std_logic;
  signal firmware_config      : std_logic_vector(7 downto 0);
  
begin

  --******************************************************************//
  -- Alternate locations for bitstreams and the history record can    //
  -- be defined if need be by setting USE_UNIQUE_CFG = "TRUE" and     //
  -- modifying those parameters                                       //
  --******************************************************************//

  unique_gen : if (USE_UNIQUE_CFG = "TRUE") generate
  begin
     NUMBER_OF_BITSTREAMS <= X"03";
     BITSTREAM_OFFSET_1   <= X"080000";
     BITSTREAM_OFFSET_2   <= X"100000";
     BITSTREAM_OFFSET_3   <= X"180000";
     HISTORY_OFFSET       <= X"070000"; 
  end generate unique_gen;

  m25p16_gen : if (SPI_DEVICE = "M25P16") generate
  begin
     NUMBER_OF_BITSTREAMS <= X"03";
     BITSTREAM_OFFSET_1   <= X"080000";
     BITSTREAM_OFFSET_2   <= X"100000";
     BITSTREAM_OFFSET_3   <= X"180000";
     HISTORY_OFFSET       <= X"070000"; 
  end generate m25p16_gen;

  at45db161d_gen : if (SPI_DEVICE = "AT45DB161D") generate
  begin
     NUMBER_OF_BITSTREAMS <= X"03";
     BITSTREAM_OFFSET_1   <= X"100000";
     BITSTREAM_OFFSET_2   <= X"200000";
     BITSTREAM_OFFSET_3   <= X"300000";
     HISTORY_OFFSET       <= X"3FFC00"; 
  end generate at45db161d_gen;

  xc3s50an_gen : if (SPI_DEVICE = "3S50AN") generate
  begin
     NUMBER_OF_BITSTREAMS <= X"01";
     BITSTREAM_OFFSET_1   <= X"020000";
     BITSTREAM_OFFSET_2   <= X"000000";
     BITSTREAM_OFFSET_3   <= X"000000";
     HISTORY_OFFSET       <= X"03FE00"; 
  end generate xc3s50an_gen;

  xc3s200an_gen : if (SPI_DEVICE = "3S200AN") generate
  begin
     NUMBER_OF_BITSTREAMS <= X"01";
     BITSTREAM_OFFSET_1   <= X"080000";
     BITSTREAM_OFFSET_2   <= X"000000";
     BITSTREAM_OFFSET_3   <= X"000000";
     HISTORY_OFFSET       <= X"0FFE00"; 
  end generate xc3s200an_gen;

  xc3s400an_gen : if (SPI_DEVICE = "3S400AN") generate
  begin
     NUMBER_OF_BITSTREAMS <= X"01";
     BITSTREAM_OFFSET_1   <= X"080000";
     BITSTREAM_OFFSET_2   <= X"000000";
     BITSTREAM_OFFSET_3   <= X"000000";
     HISTORY_OFFSET       <= X"0FFE00"; 
  end generate xc3s400an_gen;
  
  xc3s700an_gen : if (SPI_DEVICE = "3S700AN") generate
  begin
     NUMBER_OF_BITSTREAMS <= X"01";
     BITSTREAM_OFFSET_1   <= X"100000";
     BITSTREAM_OFFSET_2   <= X"000000";
     BITSTREAM_OFFSET_3   <= X"000000";
     HISTORY_OFFSET       <= X"1FFE00"; 
  end generate xc3s700an_gen; 

  xc3s1400an_gen : if (SPI_DEVICE = "3S1400AN") generate
  begin
     NUMBER_OF_BITSTREAMS <= X"01";
     BITSTREAM_OFFSET_1   <= X"200000";
     BITSTREAM_OFFSET_2   <= X"000000";
     BITSTREAM_OFFSET_3   <= X"000000";
     HISTORY_OFFSET       <= X"3FFC00"; 
  end generate xc3s1400an_gen; 

  alternate_config_process : process(NUMBER_OF_BITSTREAMS, BITSTREAM_OFFSET_1,
        BITSTREAM_OFFSET_2, BITSTREAM_OFFSET_3, HISTORY_OFFSET, port_id )
  begin
    case port_id(3 downto 0) is
      when "0000" => alternate_config <= NUMBER_OF_BITSTREAMS;
      when "0001" => alternate_config <= BITSTREAM_OFFSET_1(23 downto 16);
      when "0010" => alternate_config <= BITSTREAM_OFFSET_2(23 downto 16);
      when "0011" => alternate_config <= BITSTREAM_OFFSET_3(23 downto 16);
      when "0110" => alternate_config <= HISTORY_OFFSET(23 downto 16);
      when "0111" => alternate_config <= HISTORY_OFFSET(15 downto 8);
      when others => alternate_config <= (others => '0');
    end case;
  end process alternate_config_process;

  --******************************************************************//
  -- The picoblaze reset is the logical OR of an external reset or    // 
  -- a register command based reset                                   //
  --******************************************************************//
    
  reset_i <= RESET or reset_command;

  --******************************************************************//
  -- Instantiate SPI_ACCESS for internal FLASH (if necessary)         // 
  -- otherwise connect the spi signals to the output ports            //
  --******************************************************************//


  int_spi_gen : if (SPI_DEVICE = "3S50AN"  or SPI_DEVICE = "3S200AN" or
                    SPI_DEVICE = "3S400AN" or SPI_DEVICE = "3S700AN" or
                    SPI_DEVICE = "3S1400AN" ) generate
  begin
    SPI_MOSI  <= '1';
    SPI_SCK   <= '1';
    SPI_SS_B  <= '1';
    internal_flash <= '1';

    spiAccess_inst : SPI_ACCESS
      generic map (
        SIM_DEVICE => SPI_DEVICE )
      port map (
        MOSI => spi_mosi_i,
        MISO => spi_miso_i,
        CSB  => spi_ss_b_i,
        CLK  => spi_sck_i );
  end generate int_spi_gen;

  ext_spi_gen : if (not (SPI_DEVICE = "3S50AN"  or SPI_DEVICE = "3S200AN" or
                    SPI_DEVICE = "3S400AN" or SPI_DEVICE = "3S700AN" or
                    SPI_DEVICE = "3S1400AN" )) generate
  begin
    SPI_MOSI       <= spi_mosi_i;
    SPI_SCK        <= spi_sck_i;
    SPI_SS_B       <= spi_ss_b_i;
    spi_miso_i     <= SPI_MISO;
    internal_flash <= '0';
  end generate ext_spi_gen;
        
  --******************************************************************//
  -- Instantiate PicoBlaze and the Program ROM.                       // 
  --******************************************************************//

  kcpsm3_inst : KCPSM3 
    port map (
      address       => address,
      instruction   => instruction,
      port_id       => port_id,
      write_strobe  => write_strobe,
      out_port      => out_port,
      read_strobe   => read_strobe,
      in_port       => in_port,
      interrupt     => interrupt,
      interrupt_ack => open,
      reset         => reset_i,
      clk           => CLK );

  picocode_inst : PICOCODE
    port map (
      address       => address,
      instruction   => instruction,
      clk           => clk );

  --******************************************************************//
  -- Implement Register interface                                     //
  -- 00h rd = rx data                                                 //
  -- 00h wr = tx data                                                 //
  -- 01h rd = status                                                  //
  --          bit 7 -- tx buffer data present                         //
  --          bit 4 -- tx buffer full                                 //
  --          bit 3 -- rx buffer full                                 //
  --          bit 0 -- rx buffer data present                         //
  --          others not implemented                                  //
  -- 01h wr = not implemented                                         //
  --******************************************************************//

  regif_gen : if (USE_REG = "TRUE") generate
  begin
    reg_rd_process : process(CLK)
    begin
      if rising_edge(CLK) then
        if (read_strobe = '1' and port_id = X"02") then
          reg_rd <= '1';
        else 
          reg_rd <= '0';
        end if;
      end if;
    end process reg_rd_process;
      
    reg_wr <= '1' when write_strobe = '1' and port_id = X"02" else '0';

    regInst : PICOREGS 
      port map (
        CLK                  => CLK,
        APP_ADDR             => APP_ADDR,
        APP_DATAIN           => APP_DATAIN,
        APP_DATAOUT          => APP_DATAOUT,
        APP_RD               => APP_RD,
        APP_WE               => APP_WE,
        RESET_BUFFER         => reset_i,
        READ_BUFFER          => reg_rd,
        WRITE_BUFFER         => reg_wr,
        DATA_IN              => out_port,
        TBUFFER_DATA_PRESENT => reg_tx_pres,
        TBUFFER_HALF_FULL    => open,
        TBUFFER_FULL         => reg_tx_full,
        DATA_OUT             => reg_rx_data,
        RBUFFER_DATA_PRESENT => reg_rx_pres,
        RBUFFER_HALF_FULL    => open,
        RBUFFER_FULL         => reg_rx_full );

  end generate regif_gen;

  noregif_gen : if (not (USE_REG = "TRUE")) generate
  begin
    reg_tx_full <= '0';
    reg_tx_pres <= '0';
    reg_rx_full <= '0';
    reg_rx_pres <= '0';
    reg_rx_data <= X"00";
  end generate noregif_gen;

  --******************************************************************//
  -- Implement JTAG interface                                         //
  -- 10h rd = rx data                                                 //
  -- 10h wr = tx data                                                 //
  -- 11h rd = status                                                  //
  --          bit 7 -- tx buffer data present                         //
  --          bit 4 -- tx buffer full                                 //
  --          bit 3 -- rx buffer full                                 //
  --          bit 0 -- rx buffer data present                         //
  --          others not implemented                                  //
  -- 11h wr = not implemented                                         //
  --******************************************************************//

  jtagif_gen : if (USE_JTAG = "TRUE") generate
  begin
    jtag_rd_process : process(CLK)
    begin
      if rising_edge(CLK) then
        if (read_strobe = '1' and port_id = X"10") then
          jtag_rd <= '1';
        else 
          jtag_rd <= '0';
        end if;
      end if;
    end process jtag_rd_process;
      
    jtag_wr <= '1' when write_strobe = '1' and port_id = X"10" else '0';

    jtagInst : PICOJTAG
      port map (
        CLK                  => CLK,
        RESET_BUFFER         => reset_i,
        READ_BUFFER          => jtag_rd,
        WRITE_BUFFER         => jtag_wr,
        DATA_IN              => out_port,
        TBUFFER_DATA_PRESENT => jtag_tx_pres,
        TBUFFER_HALF_FULL    => open,
        TBUFFER_FULL         => jtag_tx_full,
        DATA_OUT             => in_port,
        RBUFFER_DATA_PRESENT => jtag_rx_pres,
        RBUFFER_HALF_FULL    => open,
        RBUFFER_FULL         => jtag_tx_full );

  end generate jtagif_gen;

  nojtagif_gen : if (not (USE_REG = "TRUE")) generate
  begin
    jtag_tx_full <= '0';
    jtag_tx_pres <= '0';
    jtag_rx_full <= '0';
    jtag_rx_pres <= '0';
    jtag_rx_data <= X"00";
  end generate nojtagif_gen;

  --******************************************************************//
  -- Implement DCE serial port, 115200-8N1                            //
  -- 20h rd = rx data                                                 //
  -- 20h wr = tx data                                                 //
  -- 21h rd = status                                                  //
  --          bit 7 -- tx buffer data present                         //
  --          bit 4 -- tx buffer full                                 //
  --          bit 3 -- rx buffer full                                 //
  --          bit 0 -- rx buffer data present                         //
  --          others not implemented                                  //
  -- 21h wr = not implemented                                         //
  --******************************************************************//

  dceif_gen : if (USE_REG = "TRUE") generate
  begin
    dce_rd_process : process(CLK)
    begin
      if rising_edge(CLK) then
        if (en_16_x_baud = '1') then 
           en_16_x_cntr <= (others => '0');
        else
           en_16_x_cntr <= en_16_x_cntr + '1';
        end if;
        if (read_strobe = '1' and port_id = X"20") then
          dce_rd <= '1';
        else 
          dce_rd <= '0';
        end if;
      end if;
    end process dce_rd_process;
      
    dce_wr <= '1' when write_strobe = '1' and port_id = X"20" else '0';
    en_16_x_baud <= '1' when (en_16_x_cntr = "1101") else '0';

    uart_tx_dce : UART_TX
      port map (
        en_16_x_baud   => en_16_x_baud,
        serial_out     => RS232_DCE_TXD,
        reset_buffer   => reset_i,
        write_buffer   => dce_wr,
        data_in        => out_port,
        buffer_half_full => open,
        buffer_full    => dce_tx_full,
        clk            => CLK );

    uart_rx_dce : UART_RX
      port map (

        en_16_x_baud   => en_16_x_baud,
        serial_in      => RS232_DCE_RXD,
        reset_buffer   => reset_i,
        read_buffer    => dce_rd,
        data_out       => dce_rx_data,
        buffer_data_present => dce_rx_pres,
        buffer_half_full => open,
        buffer_full    => dce_rx_full,
        clk            => CLK );

  end generate dceif_gen;

  nodceif_gen : if (not (USE_REG = "TRUE")) generate
  begin
    dce_tx_full <= '0';
    dce_rx_full <= '0';
    dce_rx_pres <= '0';
    dce_rx_data <= X"00";
  end generate nodceif_gen;

  --******************************************************************//
  -- Implement SPI port                                               //
  --******************************************************************//
  spireg_rd <= '1' when read_strobe = '1' and port_id(7 downto 4) = X"3"
              else '0';
  spireg_we <= '1' when write_strobe = '1' and port_id(7 downto 4) = X"3"
              else '0';

  spi_Inst : SPI
    port map (
      CLK          => CLK,
      RESET        => reset_i,
      PICO_ADDR    => port_id(2 downto 0),
      PICO_DATAIN  => out_port,
      PICO_DATAOUT => spireg_dataout,
      PICO_WE      => spireg_we,
      PICO_RD      => spireg_rd,
      MISO         => spi_miso_i,
      MOSI         => spi_mosi_i,
      SCK          => spi_sck_i,
      SS_B         => spi_ss_b_i );
  
  --******************************************************************//
  -- Instantiate the multiboot module.  Of course, multiboot          //
  -- could have been implemented completely in software, but this     //
  -- provided the opportunity to build a hardware module that could   //
  -- be used in other designs -- with or without processors.          //
  -- 40h rd = mb_int_addr[ 7: 0]                                      //
  -- 40h wr = mb_int_addr[ 7: 0]                                      //
  -- 41h rd = mb_int_addr[15: 8]                                      //
  -- 41h wr = mb_int_addr[15: 8]                                      //
  -- 42h rd = mb_int_addr[23:16]                                      //
  -- 42h wr = mb_int_addr[23:16]                                      //
  -- 43h rd = mb_int_addr[31:24]                                      //
  -- 43h wr = mb_int_addr[31:24]                                      //
  -- 44h rd = mb_int_mode[2:0]                                        //
  -- 44h wr = mb_int_mode[2:0]                                        //
  -- 45h rd = mb_int_vsel[2:0]                                        //
  -- 45h wr = mb_int_vsel[2:0]                                        //
  -- 46h rd = mb_int_use                                              //
  -- 46h wr = mb_int_use                                              //
  -- 47h rd = unused                                                  //
  -- 47h wr = multiboot strobe                                        //
  --******************************************************************//

  mbreg_process : process(CLK)
  begin
    if rising_edge(CLK) then
      if (write_strobe = '1') then
        case (port_id) is 
          when X"40" => mb_int_addr( 7 downto  0) <= out_port;
          when X"41" => mb_int_addr(15 downto  8) <= out_port;
          when X"42" => mb_int_addr(23 downto 16) <= out_port;
          when X"43" => mb_int_addr(31 downto 24) <= out_port;
          when X"44" => mb_int_mode <= out_port(2 downto 0);
          when X"45" => mb_int_vsel <= out_port(2 downto 0);
          when X"46" => mb_int_use  <= out_port(0);
          when others => null;
        end case;
      end if;
    end if;
  end process mbreg_process;

  reboot <= '1' when write_strobe = '1' and port_id = X"47" else '0';

  picoboot_inst : PICOBOOT 
    port map (
      internal_addr => mb_int_addr,
      internal_mode => mb_int_mode,
      internal_vsel => mb_int_vsel,
      internal_use  => mb_int_use,
      reboot        => reboot,
      clk           => CLK );

  --******************************************************************//
  -- Implement CRC16
  --******************************************************************//

  resetcrc <= '1' when write_strobe = '1' and port_id = X"60" else '0';
  usedata  <= '1' when write_strobe = '1' and port_id = X"61" else '0';

  crc16_gen : if (USE_CRC = "TRUE") generate
    picocrc_inst : PICOCRC 
      port map (
        CLK      => CLK,
        RESETCRC => resetcrc,
        USEDATA  => usedata,
        DIN      => out_port,
        COUT     => crc );
  end generate crc16_gen;

  nocrc16_gen : if (not (USE_CRC = "TRUE")) generate
    crc <= X"0000";
  end generate nocrc16_gen;

  --******************************************************************//
  -- Reset command
  --******************************************************************//

  reset_command <= '1' when write_strobe = '1' and port_id = X"FF" else '0';

  --******************************************************************//
  -- PROG_B command
  --******************************************************************//
  
  progb_process : process(CLK)
  begin
    if rising_edge(CLK) then
      if (port_id = X"50" and write_strobe = '1') then
        if (out_port = X"FF") then
          PROG_B <= '1';
        else
          PROG_B <= '0';
        end if;
      end if;
    end if;
  end process progb_process;

  --******************************************************************//
  -- Implement input port muxing logic.                               //
  --******************************************************************//

  firmware_config(7 downto 4) <= (others => '0');
  firmware_config(2)          <= internal_flash;
  firmware_config(1)          <= pause;

  crc16_gen2 : if (USE_CRC = "TRUE") generate
    firmware_config(3) <= '1';
  end generate crc16_gen2;

  nocrc16_gen2 : if (not (USE_CRC = "TRUE")) generate
    firmware_config(3) <= '0';
  end generate nocrc16_gen2;

  bootstrap_gen : if (BOOTSTRAP_FW = "TRUE") generate
    firmware_config(0) <= '1';
  end generate bootstrap_gen;

  application_gen : if (not (BOOTSTRAP_FW = "TRUE")) generate
    firmware_config(0) <= '0';
  end generate application_gen;

  readmux_process : process(port_id, FPGA_REVISION, reg_rx_data, reg_tx_pres,
      reg_tx_full, reg_rx_full, reg_rx_pres, jtag_rx_data, jtag_tx_pres,
      jtag_tx_full, jtag_rx_full, jtag_rx_pres, dce_rx_data, 
      dce_tx_full, dce_rx_full, dce_rx_pres, spireg_dataout, mb_int_addr,
      mb_int_mode, mb_int_vsel, mb_int_use, firmware_config, crc, 
      alternate_config)
  begin
    case (port_id) is
      when X"00" => in_port <= FPGA_REVISION(7 downto 0);
      when X"01" => in_port <= FPGA_REVISION(15 downto 8);
      when X"02" => in_port <= reg_rx_data;
      when X"03" => in_port <= reg_tx_pres & "00" & reg_tx_full &
                               reg_rx_full & "00" & reg_rx_pres;
      when X"10" => in_port <= jtag_rx_data;
      when X"11" => in_port <= jtag_tx_pres & "00" & jtag_tx_full &
                               jtag_rx_full & "00" & jtag_rx_pres;
      when X"20" => in_port <= dce_rx_data;
      when X"21" => in_port <= "000" & dce_tx_full &
                               dce_rx_full & "00" & dce_rx_pres;
      when X"30" => in_port <= spireg_dataout;
      when X"31" => in_port <= spireg_dataout;
      when X"40" => in_port <= mb_int_addr(7 downto 0);
      when X"41" => in_port <= mb_int_addr(15 downto 8);
      when X"42" => in_port <= mb_int_addr(23 downto 16);
      when X"43" => in_port <= mb_int_addr(31 downto 24);
      when X"44" => in_port <= "00000" & mb_int_mode;
      when X"45" => in_port <= "00000" & mb_int_vsel;
      when X"46" => in_port <= "0000000" & mb_int_use;
      when X"50" => in_port <= firmware_config;
      when X"61" => in_port <= crc(7 downto 0);
      when X"62" => in_port <= crc(15 downto 8);
      when X"70" => in_port <= alternate_config;
      when X"71" => in_port <= alternate_config;
      when X"72" => in_port <= alternate_config;
      when X"73" => in_port <= alternate_config;
      when X"74" => in_port <= alternate_config;
      when X"75" => in_port <= alternate_config;
      when X"76" => in_port <= alternate_config;
      when X"77" => in_port <= alternate_config;
      when others => in_port <= X"00";
    end case;
  end process readmux_process;

end RTL;
