------------------------------------------------------------------------------
-- Copyright (c) 2008 Xilinx, Inc.
-- This design is confidential and proprietary of Xilinx, All Rights Reserved.
------------------------------------------------------------------------------
--   ____  ____
--  /   /\/   /
-- /___/  \  /   Vendor:        Xilinx
-- \   \   \/    Version:       1.0.0
--  \   \        Filename:      picoCrc.vhd
--  /   /        Date Created:  April 22, 2008
-- /___/   /\    Last Modified: April 22, 2008
-- \   \  /  \
--  \___\/\___\
--
-- Devices:   Spartan-3 Generation FPGA
-- Purpose:   Failsafe multiboot  
-- Contact:   jimwe@xilinx.com
-- Reference: None
--
-- Revision History:
--   Rev 1.0.0 - (jimwe) First created April 22, 2008
--
------------------------------------------------------------------------------
--
-- LIMITED WARRANTY AND DISCLAIMER. These designs are provided to you "as is".
-- Xilinx and its licensors make and you receive no warranties or conditions,
-- express, implied, statutory or otherwise, and Xilinx specifically disclaims
-- any implied warranties of merchantability, non-infringement, or fitness for
-- a particular purpose. Xilinx does not warrant that the functions contained
-- in these designs will meet your requirements, or that the operation of
-- these designs will be uninterrupted or error free, or that defects in the
-- designs will be corrected. Furthermore, Xilinx does not warrant or make any
-- representations regarding use or the results of the use of the designs in
-- terms of correctness, accuracy, reliability, or otherwise.
--
-- LIMITATION OF LIABILITY. In no event will Xilinx or its licensors be liable
-- for any loss of data, lost profits, cost or procurement of substitute goods
-- or services, or for any special, incidental, consequential, or indirect
-- damages arising from the use or operation of the designs or accompanying
-- documentation, however caused and on any theory of liability. This
-- limitation will apply even if Xilinx has been advised of the possibility
-- of such damage. This limitation shall apply not-withstanding the failure
-- of the essential purpose of any limited remedies herein.
--
------------------------------------------------------------------------------
-- Copyright (c) 2008 Xilinx, Inc.
-- This design is confidential and proprietary of Xilinx, All Rights Reserved.
------------------------------------------------------------------------------
--
--
----------------------------------------------------------------------/
-- File:  CRC16_D8.v                             
-- Date:  Tue Nov 20 16:41:00 2007                                                      
--                                                                     
-- Copyright (C) 1999-2003 Easics NV.                 
-- This source file may be used and distributed without restriction    
-- provided that this copyright statement is not removed from the file 
-- and that any derivative work contains the original copyright notice
-- and the associated disclaimer.
--
-- THIS SOURCE FILE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS
-- OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
-- WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
--
-- Purpose: Verilog module containing a synthesizable CRC function
--   * polynomial: (0 5 12 16)
--   * data width: 8
--                                                                     
-- Info: tools@easics.be
--       http:--www.easics.com                                  
----------------------------------------------------------------------/
  

library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity PICOCRC is
  port (
    CLK           : in  std_logic;
    RESETCRC      : in  std_logic;
    USEDATA       : in  std_logic;
    DIN           : in  std_logic_vector(7 downto 0);
    COUT          : out std_logic_vector(15 downto 0)
  );
end PICOCRC;

architecture RTL of PICOCRC is

  signal C      : std_logic_vector(15 downto 0);
  signal NewCRC : std_logic_vector(15 downto 0);
  signal D      : std_logic_vector(7 downto 0);

begin

  sync_process : process(CLK)
  begin
    if rising_edge(CLK) then
      if (RESETCRC = '1') then
        C <= (others => '1');
      elsif (USEDATA = '1') then
        C <= NewCRC;
      end if;
    end if;
  end process sync_process;

  D <= DIN;
  COUT <= C;


  NewCRC(0) <= D(4) xor D(0) xor C(8) xor C(12);
  NewCRC(1) <= D(5) xor D(1) xor C(9) xor C(13);
  NewCRC(2) <= D(6) xor D(2) xor C(10) xor C(14);
  NewCRC(3) <= D(7) xor D(3) xor C(11) xor C(15);
  NewCRC(4) <= D(4) xor C(12);
  NewCRC(5) <= D(5) xor D(4) xor D(0) xor C(8) xor C(12) xor C(13);
  NewCRC(6) <= D(6) xor D(5) xor D(1) xor C(9) xor C(13) xor C(14);
  NewCRC(7) <= D(7) xor D(6) xor D(2) xor C(10) xor C(14) xor C(15);
  NewCRC(8) <= D(7) xor D(3) xor C(0) xor C(11) xor C(15);
  NewCRC(9) <= D(4) xor C(1) xor C(12);
  NewCRC(10) <= D(5) xor C(2) xor C(13);
  NewCRC(11) <= D(6) xor C(3) xor C(14);
  NewCRC(12) <= D(7) xor D(4) xor D(0) xor C(4) xor C(8) xor C(12) xor C(15);
  NewCRC(13) <= D(5) xor D(1) xor C(5) xor C(9) xor C(13);
  NewCRC(14) <= D(6) xor D(2) xor C(6) xor C(10) xor C(14);
  NewCRC(15) <= D(7) xor D(3) xor C(7) xor C(11) xor C(15);

end RTL;
